'From Squeak5.0 of 3 March 2016 [latest update: #15706] on 5 March 2016 at 6:20:02 pm'!
SharedPool subclass: #ChronologyConstants
	instanceVariableNames: 'seconds offset jdn nanos '
	classVariableNames: 'DayNames DaysInMonth MicrosecondsInDay MonthNames NanosInMillisecond NanosInSecond OneDay SecondsInDay SecondsInHour SecondsInMinute SqueakEpoch Zero '
	poolDictionaries: ''
	category: 'Chronology-Core'!

!ChronologyConstants commentStamp: 'brp 3/12/2004 14:34' prior: 0!
ChronologyConstants is a SharedPool for the constants used by the Kernel-Chronology classes.!


!DateAndTime commentStamp: 'dtl 8/4/2014 08:54' prior: 0!
I represent a point in UTC time as defined by ISO 8601. I have zero duration.


My implementation uses variables utcMicroseconds and localOffsetSeconds. This represents time magnitude as elapsed microseconds since the Posix epoch, with localOffsetSeconds representing local offset from UTC. The magnitude is used for comparison and duration calculations, and the local offset is used for displaying this magnitude in the context of a local time zone.

The implementation ignores leap seconds, which are adjustments made to maintain earth rotational clock time in synchronization with elapsed seconds.

DateAndTime class>>now will use #primitiveUtcWithOffset to obtain current time in UTC microseconds with current local offset in seconds. The primitive provides an atomic query for UTC time and local offset as measured by the OS platform.  If primitiveUtcWithOffset is not available, the traditional implementation is used, which relies on a primitive for microseconds in the local time zone and derives UTC based on the TimeZone setting.
!


!Duration commentStamp: 'dtl 7/11/2009 15:03' prior: 0!
I represent a duration of time. I have nanosecond precision!


!Stopwatch commentStamp: '<historical>' prior: 0!
A Stopwatch maintains a collection of timespans.!

Magnitude subclass: #Time
	instanceVariableNames: 'seconds nanos '
	classVariableNames: 'MillisecondClockMask '
	poolDictionaries: 'ChronologyConstants'
	category: 'Chronology-Core'!

!Time commentStamp: 'dew 10/23/2004 17:58' prior: 0!
This represents a particular point in time during any given day.  For example, '5:19:45 pm'.

If you need a point in time on a particular day, use DateAndTime.  If you need a duration of time, use Duration.
!


!TimeStamp commentStamp: '<historical>' prior: 0!
This represents a duration of 0 length that marks a particular point in time.!


!TimeZone commentStamp: 'dtl 7/11/2009 15:03' prior: 0!
TimeZone is a simple class to colect the information identifying a UTC time zone.

offset			-	Duration	- the time zone's offset from UTC
abbreviation	-	String		- the abbreviated name for the time zone.
name			-	String		- the name of the time zone.

TimeZone class >> #timeZones returns an array of the known time zones
TimeZone class >> #default returns the default time zone (Grenwich Mean Time)
!


!Timespan commentStamp: 'dtl 7/11/2009 16:35' prior: 0!
I represent a duration starting on a specific DateAndTime.!


!Date commentStamp: 'cmm 2/3/2012 17:30' prior: 0!
Instances of Date are Timespans with duration of 1 day.

Their default creation assumes a start of midnight of UTC to provide the fast, globalized Dates out of the box.  The legacy behavior that creates Timezone-sensitive Dates can be used by sending #localizedDates.
!


!Month commentStamp: 'cbr 7/28/2010 18:11' prior: 0!
I represent a month.

For example, to get the number of days this month, you can evaluate the following expression:

Month current daysInMonth!


!Schedule commentStamp: 'brp 5/13/2003 09:48' prior: 0!
I represent a powerful class for implementing recurring schedules.!


!Week commentStamp: 'cbr 7/28/2010 18:11' prior: 0!
I represent a week.

To find out what days of the week on which Squeak is fun, select the following expression, and print it:

Week dayNames!


!Year commentStamp: 'cbr 7/28/2010 18:10' prior: 0!
I represent a year.

Try me!! Select the following expression and print it:

Year current daysInYear "Try me again next leap year!!"!


!ChronologyConstants class methodsFor: 'class initialization' stamp: 'tbn 11/6/2009 11:33'!
initialize
	"ChronologyConstants initialize" 	
		
	SqueakEpoch := 2415386. 		"Julian day number of 1 Jan 1901" 
	SecondsInDay := 86400.
	SecondsInHour := 3600.
	SecondsInMinute := 60.
	NanosInSecond := 10 raisedTo: 9.
	NanosInMillisecond := 10 raisedTo: 6.
	DayNames := #(Sunday Monday Tuesday Wednesday Thursday Friday Saturday).
		
	MonthNames := #(January February March April May June July
 			August September October November December).
	DaysInMonth := #(31 28 31 30 31 30 31 31 30 31 30 31)
! !


!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/31/2014 21:10'!
+ operand
	"operand conforms to protocol Duration"

	^ self class
		utcMicroseconds: operand asDuration asNanoSeconds / 1000 + utcMicroseconds
		offset: localOffsetSeconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 12:19'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asDateAndTime)
		ifTrue: 
			[ | micros |
			micros := utcMicroseconds - operand asDateAndTime utcMicroseconds.
			Duration seconds: micros // 1000000 nanoSeconds: micros \\ 1000000 * 1000]
		ifFalse:
			[ self + (operand negated) ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 11:23'!
< comparand
	"comparand conforms to protocol DateAndTime,
	or can be converted into something that conforms."

	^utcMicroseconds < comparand asDateAndTime utcMicroseconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 8/26/2014 23:04'!
= aDateAndTimeOrTimeStamp
	"Equal if the absolute time values match, regardless of local time transform"
	| cls |
	self == aDateAndTimeOrTimeStamp ifTrue: [ ^ true ].
	cls := aDateAndTimeOrTimeStamp class.
	(cls == DateAndTime or: [ cls == TimeStamp])
		ifTrue: [^utcMicroseconds = aDateAndTimeOrTimeStamp utcMicroseconds].
	^false
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 13:11'!
asLocal
	

	^ (self offset = self class localOffset)

		ifTrue: [self]
		ifFalse: [self utcOffset: self class localOffset]! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 8/24/2014 19:23'!
asUTC

	^ localOffsetSeconds isZero
		ifTrue: [self]
		ifFalse: [self utcOffset: 0]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:03'!
dayOfMonth
	"Answer which day of the month is represented by the receiver."

	^ self
		dayMonthYearDo: [ :d :m :y | d ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 17:32'!
dayOfWeek

	"Sunday=1, ... , Saturday=7"

	^ (self julianDayNumber + 1 rem: 7) + 1
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 10:34'!
dayOfWeekAbbreviation

	^ self dayOfWeekName copyFrom: 1 to: 3
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:28'!
dayOfWeekName

	^ Week nameOfDay: self dayOfWeek! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'nice 12/26/2009 01:29'!
dayOfYear
	"This code was contributed by Dan Ingalls. It is equivalent to the terser
		^ jdn - (Year year: self year) start julianDayNumber + 1 but much quicker."

	^ self dayMonthYearDo:
		[ :d :m :y |
			| monthStart |
			monthStart := #(1 32 60 91 121 152 182 213 244 274 305 335) at: m.
			(m > 2 and: [ Year isLeapYear: y ])
				ifTrue: [ monthStart + d ]
				ifFalse: [ monthStart + d - 1 ]]! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 5/20/2014 22:10'!
hash
	^utcMicroseconds hash! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:29'!
hour

	^ self hour24! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'avi 2/21/2004 18:46'!
hour12
	"Answer an <integer> between 1 and 12, inclusive, representing the hour 
	of the day in the 12-hour clock of the local time of the receiver."
	^ self hour24 - 1 \\ 12 + 1
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
hour24


	^ (Duration seconds: self getWholeSeconds) hours! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:29'!
isLeapYear


	^ Year isLeapYear: self year
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/24/2003 11:03'!
meridianAbbreviation

	^ self asTime meridianAbbreviation
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
minute


	^ (Duration seconds: self getWholeSeconds) minutes! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:05'!
month

	^ self 
		dayMonthYearDo: [ :d :m :y | m ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:30'!
monthAbbreviation


	^ self monthName copyFrom: 1 to: 3! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:30'!
monthName


	^ Month nameOfMonth: self month! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:59'!
offset

	^ Duration seconds: localOffsetSeconds! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/23/2014 08:19'!
offset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but with its local time 
	being offset from UTC by offset."

	| newOffset newMicros |
	self flag: #FIXME. "check the definition of this and of #utcOffset:"
	newOffset := anOffset asDuration asSeconds.
	newMicros := localOffsetSeconds - newOffset * 1000000 + utcMicroseconds.
	^ self class utcMicroseconds: newMicros offset: newOffset
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
second


	^ (Duration seconds: self getWholeSeconds) seconds! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 9/4/2003 06:42'!
timeZoneAbbreviation

	^ self class localTimeZone abbreviation! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 9/4/2003 06:42'!
timeZoneName

	^ self class localTimeZone name! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:05'!
year
	^ self
		dayMonthYearDo: [ :d :m :y | y ]
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'dtl 4/10/2014 08:07'!
asSeconds
	"Return the number of seconds since the Squeak epoch"

	self flag: #FIXME. "is this whole seconds or partial? UTC or local?"

	^ utcMicroseconds / 1000000 + self class epochOffset
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 17:53'!
day

	^ self dayOfYear
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:48'!
daysInMonth
	"Answer the number of days in the month represented by the receiver."


	^ self asMonth daysInMonth! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:48'!
daysInYear

	"Answer the number of days in the year represented by the receiver."

	^ self asYear daysInYear! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 15:44'!
daysLeftInYear
	"Answer the number of days in the year after the date of the receiver."

	^ self daysInYear - self dayOfYear! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 15:44'!
firstDayOfMonth

	^ self asMonth start day
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 18:30'!
hours

	^ self hour
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 1/7/2004 15:45'!
minutes

	^ self minute
! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:50'!
monthIndex


	^ self month! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 18:31'!
seconds

	^ self second
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'klc 9/12/2010 15:56'!
asDate


	^ Date starting: self asDateAndTime! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:46'!
asDateAndTime

	^ self! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asDuration
	"Answer the duration since midnight."

	^ Duration seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asMonth

	^ Month starting: self! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:45'!
asNanoSeconds
	"Answer the number of nanoseconds since midnight"

	^ self asDuration asNanoSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asTime


	^ Time seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:20'!
asTimeStamp

	^ self
		asDateAndTime "FIXME LX hack for test support"
		as: TimeStamp! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'bf 8/28/2008 13:45'!
asUnixTime
	"answer number of seconds since unix epoch (midnight Jan 1, 1970, UTC)"

	^(self - self class unixEpoch) asSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asWeek

	^ Week starting: self! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asYear

	^ Year starting: self! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:32'!
dayMonthYearDo: aBlock
	"Evaluation the block with three arguments: day month, year."

	| l n i j dd mm yyyy |
	l := self julianDayNumber + 68569.
	n := 4 * l // 146097.
	l := l - (146097 * n + 3 // 4).
	i := 4000 * (l + 1) // 1461001.
	l := l - (1461 * i // 4) + 31.
	j := 80 * l // 2447.
	dd := l - (2447 * j // 80).
	l := j // 11.
	mm := j + 2 - (12 * l).
	yyyy := 100 * (n - 49) + i + l.

	^ aBlock
		value: dd
		value: mm
		value: yyyy! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:49'!
duration

	^ Duration zero! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/21/2014 21:40'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 13:50'!
julianDayNumber

	| posixDays posixSeconds localSeconds negativeDays |
	posixSeconds := utcMicroseconds // 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	negativeDays := 0.
	localSeconds < 0 ifTrue: [ "normalize"
			negativeDays := localSeconds // SecondsInDay.
			localSeconds := negativeDays * SecondsInDay + localSeconds].
	posixDays := localSeconds // SecondsInDay.
	^posixDays + PosixEpochJulianDays - negativeDays.
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:59'!
makeUTC
	"Make the receiver's timezone UTC."
	localOffsetSeconds := 0! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:49'!
middleOf: aDuration
	"Return a Timespan where the receiver is the middle of the Duration"

	| duration |
	duration := aDuration asDuration.

	^ Timespan starting: (self - (duration / 2)) duration: duration
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:33'!
midnight
	"Answer a DateAndTime starting at midnight of the same timezone offset as the receiver."
	^ self class basicNew
		setJdn: self julianDayNumber
		seconds: 0
		nano: 0
		offset: self offset! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/4/2014 20:19'!
nanoSecond

	^utcMicroseconds \\ 1000000 * 1000
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:49'!
noon
	"Answer a DateAndTime starting at noon"

	^ self dayMonthYearDo: 
		[ :d :m :y | self class year: y month: m day: d hour: 12 minute: 0 second: 0 ]
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 14:18'!
printHMSOn: aStream
	"Print just hh:mm:ss"

	| d |
	d := Duration seconds: self getWholeSeconds.
	aStream
		nextPutAll: (d hours asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (d minutes asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (d seconds asString padded: #left to: 2 with: $0).
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:38'!
printOn: aStream
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	Prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)"

	^self printOn: aStream withLeadingSpace: false
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 8/25/2014 19:55'!
printOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	"

	| nano loAbs loHours loMinutes loSeconds rem |
	self printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	(nano := self nanoSecond) ~= 0 ifTrue:
		[ | z ps |
		ps := nano asInteger printString padded: #left to: 9 with: $0.
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		(z > 0) ifTrue: [aStream nextPut: $.].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ].
	loAbs := localOffsetSeconds abs.
	loHours := loAbs // 3600.
	rem := loAbs \\ 3600.
	loMinutes :=rem // 60.
	loSeconds := rem \\ 60.
	aStream
		nextPut: (localOffsetSeconds positive ifTrue: [$+] ifFalse: [$-]);
		nextPutAll: (loHours asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (loMinutes asString padded: #left to: 2 with: $0).
	loSeconds = 0 ifFalse:
		[ aStream
			nextPut: $:;
			nextPutAll: (loSeconds truncated asString) ].
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:29'!
printYMDOn: aStream
	"Print just YYYY-MM-DD part.
	If the year is negative, prints out '-YYYY-MM-DD'."

	^self printYMDOn: aStream withLeadingSpace: false.
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:29'!
printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print just the year, month, and day on aStream.

	If printLeadingSpaceToo is true, then print as:
		' YYYY-MM-DD' (if the year is positive) or '-YYYY-MM-DD' (if the year is negative)
	otherwise print as:
		'YYYY-MM-DD' or '-YYYY-MM-DD' "

	| year month day |
	self dayMonthYearDo: [ :d :m :y | year := y. month := m. day := d ].
	year negative
		ifTrue: [ aStream nextPut: $- ]
		ifFalse: [ printLeadingSpaceToo ifTrue: [ aStream space ]].
	aStream
		nextPutAll: (year abs asString padded: #left to: 4 with: $0);
		nextPut: $-;
		nextPutAll: (month asString padded: #left to: 2 with: $0);
		nextPut: $-;
		nextPutAll: (day asString padded: #left to: 2 with: $0)
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/20/2014 22:28'!
setJdn: jdn seconds: s nano: n offset: o

	localOffsetSeconds :=  o asSeconds.
	utcMicroseconds := self
				posixMicrosecondsJdn: jdn
				seconds: s - localOffsetSeconds
				nanos: n
				offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:50'!
to: anEnd
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ Timespan starting: self ending: (anEnd asDateAndTime)
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 15:57'!
to: anEnd by: aDuration
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (Schedule starting: self ending: (anEnd asDateAndTime))
		schedule: (Array with: aDuration asDuration);
		yourself
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 16:01'!
to: anEnd by: aDuration do: aBlock
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (self to: anEnd by: aDuration) scheduleDo: aBlock! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/23/2014 08:13'!
utcOffset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but offset from UTC by anOffset"

	self flag: #FIXME. "check the definition of this and of #offset:"
	^self utcMicroseconds: utcMicroseconds offset: anOffset asDuration asSeconds
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:38'!
hasEqualTicks: aDateAndTime
	
	^ (self julianDayNumber = aDateAndTime julianDayNumber)
		and: [ (self getSeconds = aDateAndTime secondsSinceMidnight)
			and: [ self nanoSecond = aDateAndTime nanoSecond ] ]

! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 4/9/2014 22:24'!
normalize: i ticks: ticks base: base

	| tick div quo rem |
	tick := ticks at: i.
	div := tick asInteger digitDiv: base neg: tick negative.
	quo := (div at: 1) normalize.
	rem := (div at: 2) normalize.
	rem < 0 ifTrue: [ quo := quo - 1. rem := base + rem ].
	ticks at: (i-1) put: ((ticks at: i-1) + quo).
	ticks at: i put: rem
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 18:00'!
secondsSinceMidnight

	^ self getSeconds! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:39'!
ticks
	"Private - answer an array with our instance variables. Assumed to be UTC "

	^ Array with: self julianDayNumber with: self getSeconds with: self nanoSecond
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 23:00'!
ticks: ticks offset: utcOffset
	"ticks is {julianDayNumber. secondCount. nanoSeconds}"

	| jdn s nanos |
	self normalize: 3 ticks: ticks base: NanosInSecond.
	self normalize: 2 ticks: ticks base: SecondsInDay.

	jdn	:= ticks at: 1.
	s := ticks at: 2.
	nanos := ticks at: 3.
	localOffsetSeconds := utcOffset asSeconds.
	utcMicroseconds := self posixMicrosecondsJdn: jdn seconds: s nanos: nanos offset: localOffsetSeconds.
! !

!DateAndTime methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:49'!
setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self printString;
			yourself)! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^ aGeneralMapper visitHookPrimitive: self
! !

!DateAndTime methodsFor: 'converting' stamp: 'dtl 3/23/2014 14:56'!
asPosixSeconds

	^ (self - (self class posixEpoch offset: self offset)) asSeconds! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 4/9/2014 22:21'!
getSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^localSeconds - (posixDays * SecondsInDay).
! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 4/18/2014 13:33'!
getWholeSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds // 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := localSeconds // SecondsInDay.
	^localSeconds - (posixDays * SecondsInDay).
! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 3/23/2014 22:59'!
offsetSeconds

	^localOffsetSeconds ifNil: [localOffsetSeconds := self offset asSeconds]! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 3/23/2014 22:51'!
utcMicroseconds
	^utcMicroseconds! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 3/28/2014 07:53'!
posixEpochJulianDays

	^self class daysFromSmalltalkEpochToPosixEpoch + SqueakEpoch! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 4/18/2014 13:22'!
posixMicrosecondsJdn: j seconds: s nanos: n offset: o

	| days totalSeconds micros |
	days := j - PosixEpochJulianDays.
	totalSeconds := days * 24 * 60 * 60 + s.
	micros := totalSeconds * 1000000.
	^micros + (n / 1000)
! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 3/24/2014 16:35'!
utcMicroseconds: microsecondsSincePosixEpoch offset: tzOffset

	utcMicroseconds := microsecondsSincePosixEpoch.
	localOffsetSeconds := tzOffset.
! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 11/10/2004 05:43'!
asPointInTime

	"DateAndTime now asPointInTime"

	^ self asPointInTimeInTimeZone: LocalTimeZone! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 11/14/2004 23:01'!
asPointInTimeInTimeZone: aTimeZone
	"The time zone is used only for determining the leap second offset for
	rotational time versus absolute time. DateAndTime is rotational time (aligned
	to the rotation of the planet), and PointInTime is absolute time (atomic clock
	seconds). An iterative method is used to ensure an acceptable result at leap
	second transitions, for which two PointInTime seconds may correspond to a
	single DateAndTime second."

	| pt leap newTime |
	pt := PointInTime fromAbsoluteSeconds: jdn - SqueakEpoch * SecondsInDay
		- PointInTime posixOffset
		+ seconds - offset asSeconds + (nanos / 1000000000).
	(1 to: 10)
		inject: pt
		into: [:time :i |
			leap := aTimeZone leapSecondsAt: time.
			newTime := PointInTime fromAbsoluteSeconds: pt absoluteTime + leap.
			(newTime = time)
				ifTrue: [^ time].
			newTime].
	self notify: 'algorithm does not converge'.
	"Lack of convergence may occur if there has been a negative leap second.
	As of October 2004, there had never been a negative leap second, so if you
	are reading this now, please check the method and figure out if it is correct.
	It should be OK to just answer the value of newTime."
	^ newTime! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 3/10/2007 13:58'!
inTimeZoneNamed: tzName

	"DateAndTime now inTimeZoneNamed: #CET "
	"DateAndTime now inTimeZoneNamed: #PST8PDT "
	"DateAndTime now inTimeZoneNamed: 'Australia/Perth' "

	^ self asPointInTime asDateAndTimeForTimeZoneNamed: tzName
! !

!DateAndTime methodsFor: '*ztimestamp' stamp: 'SvenVanCaekenberghe 3/29/2014 00:43'!
asZTimestamp
	"Convert the receiver to a ZTimestamp.
	Note that we are implicitly converting to UTC by ignoring the offset.
	Note also that nano seconds are dropped."

	^ ZTimestamp new
		jdn: julianDayNumber 
		secs: seconds! !

!DateAndTime methodsFor: '*ztimestamp' stamp: 'SvenVanCaekenberghe 3/28/2014 17:08'!
subtractFromTimestamp: timestamp
	"Part of the double dispatch implementation of ZTimestamp>>#-"
	
	^ timestamp subtractTimestamp: self asZTimestamp! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak46' stamp: 'MaxLeske 2/26/2013 23:08'!
fuelSet: julianDay nanoSecond: nanoSeconds seconds: numberOfSeconds offset: anOffset
	jdn := julianDay.
	nanos := nanoSeconds.
	seconds := numberOfSeconds.
	offset := anOffset.! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak46' stamp: 'MaxLeske 2/26/2013 23:07'!
julianDayNumberUTC
	^ jdn! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak46' stamp: 'MaxLeske 2/18/2013 22:36'!
serializeOn: anEncoder

	anEncoder 
		encodeUint32: self julianDayNumberUTC;
		encodeUint32: self nanoSecond;
		encodeInt24: self secondsSinceMidnight;
		encodeInt24: self offset asSeconds;
		encodeInt32: self offset nanoSeconds.! !

!DateAndTime methodsFor: 'objects from disk' stamp: 'dtl 10/5/2014 14:46'!
convertOldFormatDataFrom: aDataStream
	"The serialized instance is a DateAndTime in the traditional format. Convert
			the stored instance variables to a UTC based DateAndTime."

	| julianDay seconds nanoSeconds offset |
	seconds := aDataStream next.
	offset := aDataStream next.
	julianDay := aDataStream next.
	nanoSeconds := aDataStream next.
	self setJdn: julianDay seconds: seconds nano: nanoSeconds offset: offset! !

!DateAndTime methodsFor: 'objects from disk' stamp: 'dtl 10/5/2014 14:47'!
readDataFrom: aDataStream size: varsOnDisk
	"Handle the special case of a DateAndTime that was saved in the traditional format
	with four instance varables."

	varsOnDisk = 4
		ifTrue: [ self convertOldFormatDataFrom: aDataStream]
		ifFalse: [ super readDataFrom: aDataStream size: varsOnDisk ]
! !

!DateAndTime methodsFor: '*ma-core' stamp: 'cmm 8/13/2014 16:14'!
maPrintAbbreviatedOn: aStream 
	| date |
	(date := self asDate) makeUTC = Date today
		ifTrue:
			[ self asTime
				print24: false
				showSeconds: false
				on: aStream ]
		ifFalse:
			[ aStream
				 maPrint: ((date printFormat: #(2 1 3 $/ 1 2 1 )) copyUpToLast: $/) ;
				 maPrint: $@.
			self asTime
				print24: false
				showSeconds: false
				on: aStream ]! !

!DateAndTime methodsFor: '*magma-client' stamp: 'cmm 10/28/2010 10:38'!
isImmutableInMagma 
	"These don't change once they're created, so don't bother to record them in magma's readSet."
	^ true! !


!DateAndTime class methodsFor: 'ansi protocol' stamp: 'gk 8/31/2006 00:49'!
clockPrecision
	"One nanosecond precision"

	^ Duration seconds: 0 nanoSeconds: 1
! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'ul 5/25/2015 11:50'!
now

	self primPosixMicrosecondClockWithOffset ifNotNil: [ :timeArray |
		^self utcMicroseconds: (timeArray at: 1) offset: (timeArray at: 2) ].
	"Use old style primitive support"
	^self nowWithOffset: self localOffset
		! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 7/27/2003 15:25'!
year: year day: dayOfYear hour: hour minute: minute second: second

	^ self
		year: year
		day: dayOfYear
		hour: hour
		minute: minute
		second: second
		offset: self localOffset
! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 7/27/2003 15:28'!
year: year day: dayOfYear hour: hour minute: minute second: second offset: offset 
	"Return a DataAndTime"

	| y d |
	y := self
		year: year
		month: 1
		day: 1
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		offset: offset.

	d := Duration days: (dayOfYear - 1).

	^ y + d
! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:00'!
year: year month: month day: day hour: hour minute: minute second: second
	"Return a DateAndTime"

	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		offset: self localOffset! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:36'!
year: year month: month day: day hour: hour minute: minute second: second offset: offset

	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		offset: offset! !

!DateAndTime class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 10:42'!
fromSeconds: seconds 
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	"dtl - Implemented for compatibiity with Squeak, but this is probably wrong"

	| s uSec offset |
	self flag: #FIXME. "local versus UTC? what should we set the offset to?"
	offset := self localOffsetSeconds.
	s := seconds - self epochOffset - offset..
	uSec := s * 1000000.
	^ self utcMicroseconds: uSec offset: offset
! !

!DateAndTime class methodsFor: 'smalltalk-80' stamp: 'kph 12/11/2006 21:13'!
millisecondClockValue

	^ self clock millisecondClockValue! !

!DateAndTime class methodsFor: 'smalltalk-80' stamp: 'kph 12/9/2006 23:46'!
totalSeconds

	^ self clock totalSeconds! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:36'!
current


	^ self now! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'bf 7/3/2012 23:35'!
date: aDate time: aTime

	^ self 
		year: aDate year 
		day: aDate dayOfYear 
		hour: aTime hour 
		minute: aTime minute 
		second: aTime second
		offset: aDate start offset! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/28/2014 18:44'!
epoch
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	| uSec |
	self flag: #FIXME. "see comment in fromSeconds: "
	uSec := self epochOffsetMicros negated.
	^ self utcMicroseconds: uSec offset: self localOffsetSeconds


"
	^ self julianDayNumber: SqueakEpoch
"! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:36'!
fromString: aString


	^ self readFrom: (ReadStream on: aString)! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:14'!
julianDayNumber: anInteger 
	^ self
		julianDayNumber: anInteger
		offset: self localOffset! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:13'!
julianDayNumber: anInteger offset: aDuration 
	^ self basicNew
		ticks: anInteger days ticks
		offset: aDuration ;
		 yourself! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 9/4/2003 06:40'!
localOffset
	"Answer the duration we are offset from UTC"

	^ self localTimeZone offset
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
midnight

	^ self now midnight! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 11/6/2010 23:38'!
milliSecondsSinceMidnight
	| msm msClock |

	"This is usually only during system startup..."
	self waitForOffsets.

	msClock := self millisecondClockValue.
	msClock < LastMilliSeconds ifTrue: [ "rolled over"
		MilliSecondOffset := MilliSecondOffset + (SmallInteger maxVal // 2) + 1 ].
	LastMilliSeconds := msClock.
	[
	msm := msClock + MilliSecondOffset.
	msm >= 86400000 ] whileTrue: [
		"next day"
		LastTick := -1.
		DaysSinceEpoch := DaysSinceEpoch + 1.
		MilliSecondOffset := MilliSecondOffset - 86400000 ].
	"day rolled over sanity check"
	(LastTick = -1 and: [
		(Duration
			days: SqueakEpoch
			hours: 0
			minutes: 0
			seconds: self clock totalSeconds) days ~= DaysSinceEpoch ]) ifTrue: [
		self initializeOffsets.
		^ self milliSecondsSinceMidnight ].
	^ msm.! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/23/2014 18:53'!
new
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	^ self utcMicroseconds: self epochOffsetMicros negated offset: 0

! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
noon

	^ self now noon
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 7/4/2012 13:42'!
nowWithOffset: aDuration 
	| nanoTicks msm |
	nanoTicks := (msm := self milliSecondsSinceMidnight) * 1000000.
	"The following usually only executes at system startup."
	^ LastTick < nanoTicks
		ifTrue:
			[ LastTick := nanoTicks.
			self waitForOffsets.
			self basicNew
				setJdn: DaysSinceEpoch
				seconds: msm // 1000
				nano: msm \\ 1000 * 1000000
				offset: aDuration ]
		ifFalse:
			[ LastTickSemaphore critical:
				[ LastTick := LastTick + 1.
				self waitForOffsets.
				self basicNew
					setJdn: DaysSinceEpoch
					seconds: LastTick // 1000000000
					nano: LastTick \\ 1000000000
					offset: aDuration ] ]
" 
[ 10000 timesRepeat: [ self now. ] ] timeToRun / 10000.0 . 

If calls to DateAndTime-c-#now are within a single millisecond the semaphore code 
to ensure that (self now <= self now) slows things down considerably by a factor of about 20.

The actual speed of a single call to DateAndTime-now in milliseconds is 
demonstrated by the unguarded method below.

[ 100000 timesRepeat: [ self todayAtMilliSeconds: (self milliSecondsSinceMidnight) ] ] timeToRun / 100000.0 .  0.00494 0.00481 0.00492 0.00495
  
"! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/30/2014 23:06'!
readFrom: aStream

	| offsetSeconds ch yearMonthDay hourMinuteSecondNano offset |

	yearMonthDay := Date readYearMonthDayFrom: aStream.
	[aStream peek isDigit]
		whileFalse: [aStream next].
	hourMinuteSecondNano := Time readHourMinuteSecondNanoFrom: aStream.
	(aStream atEnd or: [('+-Z' includes: aStream peek) not])
		ifTrue: [ self flag: #FIXME.
				"Different unit tests have conflicting opinions as to whether the
				current local offset should be used as a default. However, the current
				local offset cannot be correct due to DST (offset is itself a function
				of the point in time). Therefore, defaulting to offset 0 is the only
				approach that can be done. Unit tests should be changed accordingly."
				offsetSeconds := 0.
				"offsetSeconds := self localOffset asSeconds"]
		ifFalse: [(aStream peekFor: $Z)
			ifTrue: [offsetSeconds := 0]
			ifFalse: [
				ch := aStream next.
				ch = $+ ifTrue: [ch := Character space].
				offset := Duration fromString: ch asString, '0:', aStream upToEnd, ':0'.
				offsetSeconds := offset asSeconds]].
	^ self
		year: yearMonthDay first
		month: yearMonthDay second
		day: yearMonthDay third
		hour: hourMinuteSecondNano first
		minute: hourMinuteSecondNano second
		second: hourMinuteSecondNano third
		nanoSecond: hourMinuteSecondNano fourth
		offsetSeconds: offsetSeconds


	"	'-1199-01-05T20:33:14.321-05:00' asDateAndTime
		' 2002-05-16T17:20:45.1+01:01' asDateAndTime

		' 2002-05-16T17:20:45.02+01:01' asDateAndTime

		' 2002-05-16T17:20:45.003+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0004+01:01' asDateAndTime
  		' 2002-05-16T17:20:45.00005' asDateAndTime
		' 2002-05-16T17:20:45.000006+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0000007+01:01' asDateAndTime
		' 2002-05-16T17:20:45.00000008-01:01' asDateAndTime   
		' 2002-05-16T17:20:45.000000009+01:01' asDateAndTime  
		' 2002-05-16T17:20:45.0000000001+01:01' asDateAndTime  

 		' 2002-05-16T17:20' asDateAndTime
		' 2002-05-16T17:20:45' asDateAndTime
		' 2002-05-16T17:20:45+01:57' asDateAndTime
 		' 2002-05-16T17:20:45-02:34' asDateAndTime
 		' 2002-05-16T17:20:45+00:00' asDateAndTime
		' 1997-04-26T01:02:03+01:02:3' asDateAndTime 
 	"! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
today

	^ self midnight! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/24/2003 12:19'!
tomorrow

	^ self today asDate next asDateAndTime
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'bf 8/28/2008 13:19'!
unixEpoch
	"Answer a DateAndTime representing the Unix epoch (1 January 1970, midnight UTC)"

	^ self basicNew
		ticks: #(2440588 0 0) offset: Duration zero;
		yourself.
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:53'!
year: year day: dayOfYear
	"Return a DateAndTime"

	^ self
		year: year
		day: dayOfYear
		hour: 0
		minute: 0
		second: 0
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 10/15/2009 01:05'!
year: year month: month day: day
	"Return a DateAndTime, midnight local time"
	
	^self
 		year: year
 		month: month
 		day: day
 		hour: 0
		minute: 0
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 10/15/2009 01:19'!
year: year month: month day: day hour: hour minute: minute
	"Return a DateAndTime"

	^self
 		year: year
 		month: month
 		day: day
 		hour: hour
		minute: minute
		second: 0
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 15:04'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offset: offset
	"Return a DateAndTime"

	| offsetSeconds utcMicros |
	offsetSeconds := offset asSeconds.
	utcMicros := self
				utcMicrosecondsForYear: year
				month: month
				day: day
				hour: hour
				minute: minute
				second: second
				nanoSecond: nanoCount
				offsetSeconds: offsetSeconds.
	^ self utcMicroseconds: utcMicros offset: offsetSeconds! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:12'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offsetSeconds: offsetSeconds
	"Return a DateAndTime"

	| utcMicros |
	utcMicros := self
				utcMicrosecondsForYear: year
				month: month
				day: day
				hour: hour
				minute: minute
				second: second
				nanoSecond: nanoCount
				offsetSeconds: offsetSeconds.
	^ self utcMicroseconds: utcMicros offset: offsetSeconds! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/24/2003 12:19'!
yesterday

	^ self today asDate previous asDateAndTime! !

!DateAndTime class methodsFor: 'clock provider' stamp: 'kph 12/11/2006 20:14'!
clock 
	 "the provider of real time seconds/milliseconds."

	^ ClockProvider ! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 5/26/2015 19:58'!
hasUtcPrim
	^ self primPosixMicrosecondClockWithOffset notNil
! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 4/18/2014 13:20'!
initialize

	super initialize.

	ClockProvider := Time.
	LastTickSemaphore := Semaphore forMutualExclusion.
	LastMilliSeconds := 0.
	LastTick := 0.
	PosixEpochJulianDays := 2440588.
	Smalltalk addToStartUpList: self.
	self startUp: true
! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'ul 11/6/2010 23:37'!
initializeOffsets
	| durationSinceEpoch secondsSinceMidnight nowSecs |
	LastTick := 0.
	nowSecs := self clock secondsWhenClockTicks.
	LastMilliSeconds := self millisecondClockValue.
	durationSinceEpoch := Duration
		days: SqueakEpoch
		hours: 0
		minutes: 0
		seconds: nowSecs.
	DaysSinceEpoch := durationSinceEpoch days.
	secondsSinceMidnight := (durationSinceEpoch -
		(Duration
			days: DaysSinceEpoch
			hours: 0
			minutes: 0
			seconds: 0)) asSeconds.
	MilliSecondOffset := secondsSinceMidnight * 1000 - LastMilliSeconds! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 8/26/2014 19:07'!
startUp: resuming
	resuming ifFalse: [ ^ self ].
	Time initializeMillisecondClockMask.
	self hasUtcPrim
		ifTrue: [ "Local clock rollover calculation not required"
			OffsetsAreValid := true]
		ifFalse: [ "Old procedure based on local millisecond clock"
			OffsetsAreValid := false.
			[
				self initializeOffsets.
				OffsetsAreValid := true
			] forkAt: Processor userInterruptPriority].! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'jmv 10/19/2010 14:10'!
waitForOffsets
	OffsetsAreValid ifFalse: [
		[
			(Delay forSeconds: 1) wait.
			OffsetsAreValid
		] whileFalse
	]! !

!DateAndTime class methodsFor: 'accessing' stamp: 'dtl 3/28/2014 08:05'!
localOffsetSeconds
	^self localOffset asSeconds! !

!DateAndTime class methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:39'!
localTimeZone
	"Answer the local time zone"

	^ LocalTimeZone ifNil: [ LocalTimeZone := TimeZone default ]

! !

!DateAndTime class methodsFor: 'accessing' stamp: 'nk 3/30/2004 09:53'!
localTimeZone: aTimeZone
	"Set the local time zone"

	"
	DateAndTime localTimeZone: (TimeZone offset:  0 hours name: 'Universal Time' abbreviation: 'UTC').
	DateAndTime localTimeZone: (TimeZone offset: -8 hours name: 'Pacific Standard Time' abbreviation: 'PST').
	"

	LocalTimeZone := aTimeZone


! !

!DateAndTime class methodsFor: '*Fuel' stamp: 'MaxLeske 2/18/2013 22:36'!
materializeFrom: aDecoder

	^ self basicNew
		fuelSet: aDecoder nextEncodedUint32
		nanoSecond: aDecoder nextEncodedUint32
		seconds: aDecoder nextEncodedInt24
		offset: (Duration 
			seconds: aDecoder nextEncodedInt24 
			nanoSeconds: aDecoder nextEncodedInt32);
		yourself.! !

!DateAndTime class methodsFor: 'DTL' stamp: 'ul 5/24/2015 23:39'!
primPosixMicrosecondClockWithOffset
	"Answer an array with UTC microseconds since the Posix epoch and the
	current seconds offset from GMT in the local time zone."

	<primitive: 'primitiveUtcWithOffset'>
	^nil! !

!DateAndTime class methodsFor: 'DTL' stamp: 'ul 5/25/2015 11:52'!
utcMicroseconds: microsecondsSincePosixEpoch offset: offsetSeconds

	^self basicNew
		utcMicroseconds: microsecondsSincePosixEpoch
		offset: offsetSeconds! !

!DateAndTime class methodsFor: '*time-TimeZones' stamp: 'dtl 11/2/2004 15:40'!
fromSeconds: seconds offsetSeconds: offset
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	| since offsetDuration |
	since := Duration days: SqueakEpoch hours: 0 minutes: 0 seconds: seconds.
	offsetDuration := Duration seconds: offset.
	^ self basicNew
		ticks: since ticks offset: offsetDuration;
		yourself.
! !

!DateAndTime class methodsFor: 'constants' stamp: 'dtl 8/26/2014 19:18'!
daysFromSmalltalkEpochToPosixEpoch

	^ 25202 "52 * 365 + (17 * 366)"! !

!DateAndTime class methodsFor: 'constants' stamp: 'dtl 8/26/2014 19:22'!
epochOffset
	"Elaspsed seconds from the Smalltalk epoch to the Posix epoch"
	^ 2177452800 "self daysFromSmalltalkEpochToPosixEpoch * SecondsInDay"! !

!DateAndTime class methodsFor: 'constants' stamp: 'dtl 8/26/2014 19:22'!
epochOffsetMicros
	"Elapsed microseconds from the Smalltalk epoch to the Posix epoch"
	^ 2177452800000000 "self epochOffset * 1000000"! !

!DateAndTime class methodsFor: 'constants' stamp: 'dtl 4/18/2014 13:22'!
posixEpoch
	"Answer a DateAndTime representing the Posix epoch"

	^ self julianDayNumber: PosixEpochJulianDays ! !

!DateAndTime class methodsFor: 'instance creation' stamp: 'dtl 4/18/2014 15:03'!
utcMicrosecondsForYear: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount  offsetSeconds: offsetSeconds

	| monthIndex daysInMonth p q r s julianDayNumber posixDays seconds utcSeconds |

	monthIndex := month isInteger ifTrue: [month] ifFalse: [Month indexOfMonth: month].
	daysInMonth := Month
		daysInMonth: monthIndex
		forYear: year.
	day < 1 ifTrue: [self error: 'day may not be zero or negative'].
	day > daysInMonth ifTrue: [self error: 'day is after month ends']. 	
	
	p := (monthIndex - 14) quo: 12.
	q := year + 4800 + p.
	r := monthIndex - 2 - (12 * p).
	s := (year + 4900 + p) quo: 100.

	julianDayNumber :=
 		( (1461 * q) quo: 4 ) +
			( (367 * r) quo: 12 ) -
 				( (3 * s) quo: 4 ) +
 					( day - 32075 ).

	posixDays := julianDayNumber - PosixEpochJulianDays.
	seconds := hour * 60 + minute * 60 + second - offsetSeconds.
	utcSeconds := seconds + (posixDays * 24 * 3600).
	^ utcSeconds * 1000000 + (nanoCount / 1000)
! !


!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:59'!
* operand
	"operand is a Number" 	^ self class nanoSeconds: ( (self asNanoSeconds * operand) asInteger)
! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:59'!
+ operand

	"operand is a Duration" 	^ self class nanoSeconds: (self asNanoSeconds + operand asNanoSeconds)! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:59'!
- operand
	"operand is a Duration" 	^ self + operand negated! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:00'!
/ operand

	"operand is a Duration or a Number"


	^ operand isNumber
		ifTrue: [ self class nanoSeconds: (self asNanoSeconds / operand) asInteger ]
		ifFalse: [ self asNanoSeconds / operand asDuration asNanoSeconds ]
! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:00'!
< comparand

	^ self asNanoSeconds < comparand asNanoSeconds! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 1/9/2004 06:25'!
= comparand 
	"Answer whether the argument is a <Duration> representing the same 
	period of time as the receiver."

	^ self == comparand
		ifTrue: [true]
		ifFalse: 
			[self species = comparand species 
				ifTrue: [self asNanoSeconds = comparand asNanoSeconds]
				ifFalse: [false] ]
! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
abs

	^ self class seconds: seconds abs nanoSeconds: nanos abs! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
asDuration

	^ self! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
asSeconds


	^ seconds! !

!Duration methodsFor: 'ansi protocol' stamp: 'gk 8/30/2006 23:42'!
days
	"Answer the number of days the receiver represents."

	^ seconds quo: SecondsInDay
! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
hash

 	^seconds bitXor: nanos! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
hours
	"Answer the number of hours the receiver represents."


	^ (seconds rem: SecondsInDay) quo: SecondsInHour! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:01'!
minutes

	"Answer the number of minutes the receiver represents."


	^ (seconds rem: SecondsInHour) quo: SecondsInMinute! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:02'!
negated

	^ self class seconds: seconds negated nanoSeconds: nanos negated! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:02'!
negative


	^ self positive not! !

!Duration methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:02'!
positive


	^ seconds = 0 ifTrue: [ nanos positive ] ifFalse: [ seconds positive ]! !

!Duration methodsFor: 'ansi protocol' stamp: 'KLC 5/12/2008 13:42'!
seconds
	"Answer the number of seconds the receiver represents."

	^seconds rem: SecondsInMinute! !

!Duration methodsFor: 'initialize-release' stamp: 'nk 3/30/2004 10:01'!
initialize
	self seconds: 0 nanoSeconds: 0.
! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 14:29'!
// operand

	"operand is a Duration or a Number"


	^ operand isNumber
		ifTrue: [ self class nanoSeconds: (self asNanoSeconds // operand) asInteger ]
		ifFalse: [ self asNanoSeconds // operand asDuration asNanoSeconds ]! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 15:07'!
\\ operand

	"modulo. Remainder defined in terms of //. Answer a Duration with the 
	same sign as aDuration. operand is a Duration or a Number."

	^ operand isNumber
		ifTrue: [ self class nanoSeconds: (self asNanoSeconds \\ operand) ]
		ifFalse: [ self - (operand * (self // operand)) ]! !

!Duration methodsFor: 'squeak protocol' stamp: 'cmm 4/14/2013 15:31'!
ago
	"Answer the DateAndTime which was the receiver's duration ago.
	e.g., 5 minutes ago.  2 days ago."
	^ DateAndTime now - self! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 13:42'!
asDelay

	^ Delay forDuration: self
! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:03'!
asMilliSeconds


	^ ((seconds * NanosInSecond) + nanos) // (10 raisedToInteger: 6)! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:03'!
asNanoSeconds

	^ (seconds * NanosInSecond) + nanos! !

!Duration methodsFor: 'squeak protocol' stamp: 'cmm 4/14/2013 15:32'!
fromNow
	"Answer the DateAndTime which which occurs the receiver's duration from now.
	e.g., 5 minutes fromNow.  2 days fromNow."
	^ DateAndTime now + self! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 4/13/2006 10:20'!
isZero

	^ seconds = 0 and: [ nanos = 0 ]
! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:03'!
nanoSeconds


	^ nanos! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 13:22'!
printOn: aStream
	"Format as per ANSI 5.8.2.16: [-]D:HH:MM:SS[.S]" 	| d h m s n |
	d := self days abs.
	h := self hours abs.
	m := self minutes abs.
 	s := self seconds abs truncated.
	n := self nanoSeconds abs. 	self negative ifTrue: [ aStream nextPut: $- ].
	d printOn: aStream. aStream nextPut: $:.
	h < 10 ifTrue: [ aStream nextPut: $0. ].
	h printOn: aStream. aStream nextPut: $:.
	m < 10 ifTrue: [ aStream nextPut: $0. ].
	m printOn: aStream. aStream nextPut: $:.
	s < 10 ifTrue: [ aStream nextPut: $0. ].
	s printOn: aStream.
	n = 0 ifFalse:
		[ | z ps |
		aStream nextPut: $..
		ps := n printString padded: #left to: 9 with: $0. 
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ]
! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 15:42'!
roundTo: aDuration
	"e.g. if the receiver is 5 minutes, 37 seconds, and aDuration is 2 minutes, answer 6 minutes."

	^ self class nanoSeconds: (self asNanoSeconds roundTo: aDuration asNanoSeconds)
! !

!Duration methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 15:38'!
truncateTo: aDuration
	"e.g. if the receiver is 5 minutes, 37 seconds, and aDuration is 2 minutes, answer 4 minutes."

	^ self class
		nanoSeconds: (self asNanoSeconds truncateTo: aDuration asNanoSeconds)
! !

!Duration methodsFor: 'private' stamp: 'dtl 8/29/2010 11:31'!
seconds: secondCount nanoSeconds: nanoCount 
	"Private - only used by Duration class"

	seconds := secondCount.
	nanos := nanoCount rounded.
	"normalize if signs do not match"
	[ nanos < 0 and: [ seconds > 0 ] ]
		whileTrue: [ seconds := seconds - 1.
			nanos := nanos + NanosInSecond ].
	[ seconds < 0 and: [ nanos > 0 ] ]
		whileTrue: [ seconds := seconds + 1.
			nanos := nanos - NanosInSecond ]

! !

!Duration methodsFor: 'private' stamp: 'brp 9/25/2003 14:42'!
storeOn: aStream

	aStream
		nextPut: $(;
		nextPutAll: self className;
		nextPutAll: ' seconds: ';
		print: seconds;
		nextPutAll: ' nanoSeconds: ';
		print: nanos;
		nextPut: $)
! !

!Duration methodsFor: 'private' stamp: 'gk 8/30/2006 23:41'!
ticks
	"Answer an array {days. seconds. nanoSeconds}. Used by DateAndTime and Time."

	| days |
	days := self days.
	^ Array 
		with: days
		with: seconds - (days * SecondsInDay)
		with: nanos
! !

!Duration methodsFor: '*ztimestamp' stamp: 'SvenVanCaekenberghe 3/28/2014 17:08'!
subtractFromTimestamp: timestamp
	"Part of the double dispatch implementation of ZTimestamp>>#-"
	
	^ timestamp subtractDuration: self! !

!Duration methodsFor: '*grease-pharo-core' stamp: 'jf 2/5/2010 23:58'!
asMilliseconds
	^ self asMilliSeconds! !

!Duration methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:02'!
milliseconds
	^ nanos quo: NanosInMillisecond! !

!Duration methodsFor: '*magma-client' stamp: 'cmm 11/2/2010 11:24'!
isImmutableInMagma
	^ true! !


!Duration class methodsFor: 'ansi protocol' stamp: 'gk 8/31/2006 01:09'!
days: days hours: hours minutes: minutes seconds: seconds

	^ self days: days hours: hours minutes: minutes seconds: seconds nanoSeconds: 0! !

!Duration class methodsFor: 'ansi protocol' stamp: 'gk 8/30/2006 23:18'!
days: days seconds: seconds

	^ self basicNew seconds: days * SecondsInDay + seconds nanoSeconds: 0
! !

!Duration class methodsFor: 'ansi protocol' stamp: 'gk 8/31/2006 01:34'!
seconds: seconds

	^ self seconds: seconds nanoSeconds: 0
! !

!Duration class methodsFor: 'ansi protocol' stamp: 'cmm 7/16/2011 15:13'!
zero
	"Answer the canonicalized instance of Duration zero."
	^ Zero! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:25'!
days: aNumber

	^ self seconds: aNumber * SecondsInDay nanoSeconds: 0! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:26'!
days: days hours: hours minutes: minutes seconds: seconds nanoSeconds: nanos	

 	^ self seconds: ((days * SecondsInDay) 
						+ (hours * SecondsInHour)
							+ (minutes * SecondsInMinute) 
								+ seconds)
		nanoSeconds: nanos
! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:07'!
fromString: aString

	^ self readFrom: (ReadStream on: aString)
! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:26'!
hours: aNumber

	^ self seconds: aNumber * SecondsInHour nanoSeconds: 0! !

!Duration class methodsFor: 'squeak protocol' stamp: 'ul 4/28/2010 09:44'!
milliSeconds: milliCount 
	
	^self
		seconds: (milliCount quo: 1000)
		nanoSeconds: (milliCount rem: 1000) * NanosInMillisecond! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:27'!
minutes: aNumber

	^ self seconds: aNumber * SecondsInMinute nanoSeconds: 0! !

!Duration class methodsFor: 'squeak protocol' stamp: 'brp 1/9/2004 17:20'!
month: aMonth
	"aMonth is an Integer or a String"
	
	^ (Month month: aMonth year: Year current year) duration
! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:30'!
nanoSeconds: nanos
	"This method is slow. If you have nanos less than 10^6 you should use #seconds:nanoSeconds: instead."

	| quo |
	quo := nanos quo: NanosInSecond.
	^ self basicNew
		seconds: quo
		nanoSeconds: nanos - (quo * NanosInSecond)
! !

!Duration class methodsFor: 'squeak protocol' stamp: 'cmm 7/17/2011 14:59'!
oneDay
	"Answer the canonicalized Duration representing length of 1 day.  Used by Dates."
	^ OneDay! !

!Duration class methodsFor: 'squeak protocol' stamp: 'tfel 9/1/2009 22:29'!
readFrom: aStream
	"Formatted as per ANSI 5.8.2.16: [-]D:HH:MM:SS[.S]"

	| sign days hours minutes seconds nanos nanosBuffer |
	sign := (aStream peekFor: $-) ifTrue: [-1] ifFalse: [1].
	days := (aStream upTo: $:) asInteger sign: sign.
	hours := (aStream upTo: $:) asInteger sign: sign.
	minutes := (aStream upTo: $:) asInteger sign: sign.
	seconds := (aStream upTo: $.) asInteger sign: sign.
	nanosBuffer := '000000000' copy.
	nanos := WriteStream on: nanosBuffer.
	[aStream atEnd not and: [aStream peek isDigit]]
		whileTrue: [nanos nextPut: aStream next].
		
	^ self 
		days: days 
		hours: hours 
		minutes: minutes 
		seconds: seconds 
		nanoSeconds: (nanosBuffer asInteger sign: sign)

	"	'0:00:00:00' asDuration
		'0:00:00:00.000000001' asDuration
		'0:00:00:00.999999999' asDuration
		'0:00:00:00.100000000' asDuration
		'0:00:00:00.001 ' asDuration
		'0:00:00:00.1' asDuration
		'0:00:00:01 ' asDuration
		'0:12:45:45' asDuration
		'1:00:00:00' asDuration
		'365:00:00:00' asDuration
		'-7:09:12:06.10' asDuration
		'+0:01:02:3' asDuration
 	"! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 01:32'!
seconds: seconds nanoSeconds: nanos

	^ self basicNew
		seconds: seconds truncated
		nanoSeconds: seconds fractionPart * NanosInSecond + nanos! !

!Duration class methodsFor: 'squeak protocol' stamp: 'gk 8/30/2006 23:20'!
weeks: aNumber

	^ self days: (aNumber * 7) seconds: 0
! !

!Duration class methodsFor: 'initialize-release' stamp: 'cmm 7/17/2011 14:58'!
initialize
	ChronologyConstants classPool
		at: #Zero
		put:
			(self basicNew
				seconds: 0
				nanoSeconds: 0) ;
		at: #OneDay
		put: 1 day! !

!Duration class methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:12'!
milliseconds: anInteger
	^ self milliSeconds: anInteger! !


!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 23:12'!
activate

	self isSuspended ifTrue:
		[self timespans add: 
			(Timespan starting: DateAndTime now duration: Duration zero).
		self state: #active]
! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 23:45'!
duration

	| ts last |
	self isSuspended 
		ifTrue:
			[ (ts := self timespans) isEmpty ifTrue: 
				[ ts := { Timespan starting: DateAndTime now duration: Duration zero } ] ]
		ifFalse:
			[ last := self timespans last.
			ts := self timespans allButLast
				add: (last duration: (DateAndTime now - last start); yourself);
				yourself ].
		
	^ (ts collect: [ :t | t duration ]) sum! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 11:21'!
end

	^ self timespans last next

! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 22:48'!
isActive

	^ self state = #active
! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 22:48'!
isSuspended

	^ self state = #suspended

! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 13:25'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self state;
		nextPut: $:;
		print: self duration;
		nextPut: $).

! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 12:03'!
reActivate

	self 
		suspend;
		activate.
! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 11:54'!
reset

	self suspend.
	timespans := nil
! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 23:18'!
start

	^ self timespans first start

! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 22:47'!
state

	^ state ifNil: [ state := #suspended ]! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 22:46'!
state: aSymbol

	state := aSymbol! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 23:13'!
suspend

	| ts |
	self isActive ifTrue:
		[ ts := self timespans last.
		ts duration: (DateAndTime now - ts start).
		self state: #suspended]! !

!Stopwatch methodsFor: 'squeak protocol' stamp: 'brp 9/24/2003 22:44'!
timespans

	^ timespans ifNil: [ timespans := OrderedCollection new ]! !

!Stopwatch methodsFor: 'copying' stamp: 'cmm 1/28/2012 16:31'!
postCopy
	super postCopy.
	timespans := timespans copy! !


!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 15:54'!
< aTime

	^ self asDuration < aTime asDuration! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:11'!
= aTime

	^ [ self ticks = aTime ticks ]
		on: MessageNotUnderstood do: [false]! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:32'!
duration

	^ Duration zero! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:11'!
hash

	^ self ticks hash! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:10'!
hour

	^ self hour24! !

!Time methodsFor: 'ansi protocol' stamp: 'avi 2/21/2004 18:45'!
hour12
	"Answer an <integer> between 1 and 12, inclusive, representing the hour 
	of the day in the 12-hour clock of the local time of the receiver."
	^ self hour24 - 1 \\ 12 + 1
! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:17'!
hour24


	^ self asDuration hours! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:41'!
meridianAbbreviation

	^ self hour < 12 ifTrue: ['AM'] ifFalse: ['PM']
! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 22:08'!
minute

	^ self asDuration minutes! !

!Time methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 19:19'!
second


	^ self asDuration seconds
! !

!Time methodsFor: 'printing' stamp: 'BP 5/17/2000 19:44'!
hhmm24
	"Return a string of the form 1123 (for 11:23 am), 2154 (for 9:54 pm), of exactly 4 digits"

	^(String streamContents: 
		[ :aStream | self print24: true showSeconds: false on: aStream ])
			copyWithout: $:
! !

!Time methodsFor: 'printing' stamp: 'BP 5/17/2000 19:43'!
print24
	"Return as 8-digit string 'hh:mm:ss', with leading zeros if needed"

	^String streamContents:
		[ :aStream | self print24: true on: aStream ]
! !

!Time methodsFor: 'printing' stamp: 'BP 5/17/2000 19:41'!
print24: hr24 on: aStream 
	"Format is 'hh:mm:ss' or 'h:mm:ss am' "

	self print24: hr24 showSeconds: true on: aStream! !

!Time methodsFor: 'printing' stamp: 'bf 8/31/2012 20:23'!
print24: hr24 showSeconds: showSeconds on: aStream 
	"Format is 'hh:mm:ss' or 'h:mm:ss am'  or, if showSeconds is false, 'hh:mm' or 'h:mm am'"

	| h m s |
	h := self hour. m := self minute. s := self second.
	hr24
		ifTrue: 
			[ h < 10 ifTrue: [ aStream nextPutAll: '0' ].
			h printOn: aStream ]
		ifFalse:
			[ h > 12
				ifTrue: [h - 12 printOn: aStream]
				ifFalse: 
					[h < 1
						ifTrue: [ 12 printOn: aStream ]
						ifFalse: [ h printOn: aStream ]]].

	aStream nextPutAll: (m < 10 ifTrue: [':0'] ifFalse: [':']).
	m printOn: aStream.

	showSeconds ifTrue:
		[ aStream nextPutAll: (s < 10 ifTrue: [':0'] ifFalse: [':']).
		self nanoSecond = 0
			ifTrue: [s asInteger printOn: aStream]
			ifFalse: [s asInteger * NanosInSecond + self nanoSecond asInteger 
				printOn: aStream asFixedPoint: NanosInSecond]].

	hr24 ifFalse:
		[ aStream nextPutAll: (h < 12 ifTrue: [' am'] ifFalse: [' pm']) ].
! !

!Time methodsFor: 'printing' stamp: 'tk 9/7/2000 00:09'!
printMinutes
	"Return as string 'hh:mm pm'  "

	^String streamContents:
		[ :aStream | self print24: false showSeconds: false on: aStream ]! !

!Time methodsFor: 'printing' stamp: 'dtl 6/25/2009 19:13'!
printOn: aStream 

	self print24: false
		showSeconds: (self seconds ~= 0
				or: [self nanoSecond ~= 0])
		on: aStream! !

!Time methodsFor: 'printing' stamp: 'di 9/22/2000 12:46'!
storeOn: aStream

	aStream print: self printString; nextPutAll: ' asTime'
! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 19:02'!
addSeconds: nSeconds 
	"Answer a Time that is nSeconds after the receiver."

	^ self class seconds: self asSeconds + nSeconds! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 19:02'!
addTime: timeAmount
	"Answer a Time that is timeInterval after the receiver. timeInterval is an 
	instance of Date or Time."

	^ self class seconds: self asSeconds + timeAmount asSeconds
! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 15:55'!
asSeconds
	"Answer the number of seconds since midnight of the receiver."

	^ seconds
! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:29'!
hours

	^ self hour! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 19:24'!
intervalString
	"Treat the time as a difference.  Give it in hours and minutes with two digits of accuracy."

	| d |
	d := self asDuration.
	^ String streamContents: [ :s |
		d hours > 0 ifTrue: [s print: d hours; nextPutAll: ' hours'].
		d minutes > 0 ifTrue: [s space; print: d minutes; nextPutAll: ' minutes'].
		d seconds > 0 ifTrue: [s space; print: d seconds; nextPutAll: ' seconds'] ]
! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 22:07'!
minutes

	^ self asDuration minutes! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 18:18'!
seconds

	^ self second! !

!Time methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 19:03'!
subtractTime: timeAmount 
	"Answer a Time that is timeInterval before the receiver. timeInterval is  
	an instance of Date or Time."

	^ self class seconds: self asSeconds - timeAmount asSeconds! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 23:58'!
asDate

	^ Date today! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:26'!
asDateAndTime

	^ DateAndTime today + self! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:01'!
asDuration

	"Answer the duration since midnight"

	^ Duration seconds: seconds nanoSeconds: nanos! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:29'!
asMonth

	^ self asDateAndTime asMonth! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:29'!
asNanoSeconds
	"Answer the number of nanoseconds since midnight"

	^ self asDuration asNanoSeconds! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:08'!
asTime

	^ self! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:27'!
asTimeStamp

	^ self asDateAndTime asTimeStamp! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:28'!
asWeek

	^ self asDateAndTime asWeek! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:43'!
asYear

	^ self asDateAndTime asYear! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:33'!
nanoSecond


	^ nanos! !

!Time methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:35'!
to: anEnd
	"Answer a Timespan. anEnd must respond to #asDateAndTime"

	^ self asDateAndTime to: anEnd! !

!Time methodsFor: 'private' stamp: 'gk 8/31/2006 00:20'!
seconds: secondCount
	"Private - only used by Time class."

	seconds := secondCount.
	nanos := 0
! !

!Time methodsFor: 'private' stamp: 'gk 8/31/2006 00:20'!
seconds: secondCount nanoSeconds: nanoCount 
	"Private - only used by Time class."

	seconds := secondCount.
	nanos := nanoCount
! !

!Time methodsFor: 'private' stamp: 'brp 8/23/2003 22:38'!
ticks
	"Answer an Array: { seconds. nanoSeconds }"

	^ Array with: 0 with: seconds with: nanos.! !

!Time methodsFor: 'private' stamp: 'gk 8/30/2006 23:28'!
ticks: anArray
	"ticks is an Array: { days. seconds. nanoSeconds }"

	seconds := anArray at: 2.
	nanos := anArray at: 3
! !

!Time methodsFor: '*Fuel' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^ aGeneralMapper visitHookPrimitive: self
! !

!Time methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/9/2012 14:27'!
serializeOn: anEncoder

	anEncoder 
		encodeUint24: self asSeconds;
		encodeUint32: self nanoSecond.
		! !

!Time methodsFor: '*magma-client' stamp: 'cmm 12/24/2011 16:09'!
isImmutableInMagma 
	^ true! !


!Time class methodsFor: 'ansi protocol' stamp: 'kph 12/9/2006 10:17'!
milliSecondsSinceMidnight
	
^ DateAndTime milliSecondsSinceMidnight! !

!Time class methodsFor: 'ansi protocol' stamp: 'kph 12/9/2006 10:18'!
now
	"Answer a Time representing the time right now - this is a 24 hour clock."

	| ms |
	
	ms := self milliSecondsSinceMidnight.

	^ self seconds:  (ms // 1000) nanoSeconds: (ms \\ 1000) * 1000000


! !

!Time class methodsFor: 'benchmarks' stamp: 'brp 8/24/2003 00:06'!
benchmarkMillisecondClock		"Time benchmarkMillisecondClock"

	"Benchmark the time spent in a call to Time>>millisecondClockValue.

	On the VM level this tests the efficiency of calls to ioMSecs()."

	"PII/400 Windows 98: 0.725 microseconds per call"

	| temp1 temp2 temp3 delayTime nLoops time |

	delayTime := 5000. "Time to run benchmark is approx. 2*delayTime"



	"Don't run the benchmark if we have an active delay since

	we will measure the additional penalty in the primitive dispatch

	mechanism (see #benchmarkPrimitiveResponseDelay)."

	Delay anyActive ifTrue:[

		^self notify:'Some delay is currently active.

Running this benchmark will not give any useful result.'].



	"Flush the cache for this benchmark so we will have

	a clear cache hit for each send to #millisecondClockValue below"

	Object flushCache.

	temp1 := 0.

	temp2 := self. "e.g., temp1 == Time"

	temp3 := self millisecondClockValue + delayTime.



	"Now check how often we can run the following loop in the given time"

	[temp2 millisecondClockValue < temp3]

		whileTrue:[temp1 := temp1 + 1].



	nLoops := temp1. "Remember the loops we have run during delayTime"



	"Setup the second loop"

	temp1 := 0.

	temp3 := nLoops.



	"Now measure how much time we spend without sending #millisecondClockValue"

	time := Time millisecondClockValue.

	[temp1 < temp3]

		whileTrue:[temp1 := temp1 + 1].

	time := Time millisecondClockValue - time.



	"And compute the number of microseconds spent per call to #millisecondClockValue"

	^((delayTime - time * 1000.0 / nLoops) truncateTo: 0.001) printString,

		' microseconds per call to Time>>millisecondClockValue'
! !

!Time class methodsFor: 'benchmarks' stamp: 'BP 3/30/2001 15:25'!
benchmarkPrimitiveResponseDelay	"Time benchmarkPrimitiveResponseDelay"

	"Benchmark the overhead for primitive dispatches with an active Delay.

	On the VM level, this tests the efficiency of ioLowResMSecs."



	"PII/400 Windows98: 0.128 microseconds per prim"



	"ar 9/6/1999: This value is *extremely* important for stuff like sockets etc.

	I had a bad surprise when Michael pointed this particular problem out:

	Using the hardcoded clock() call for ioLowResMSecs on Win32 resulted in an overhead

	of 157.4 microseconds per primitive call - meaning you can't get no more than

	approx. 6000 primitives per second on my 400Mhz PII system with an active delay!!

	BTW, it finally explains why Squeak seemed soooo slow when running PWS or 

	other socket stuff. The new version (not using clock() but some Windows function) 

	looks a lot better (see above; approx. 8,000,000 prims per sec with an active delay)."



	| nLoops bb index baseTime actualTime delayTime |

	delayTime := 5000. "Time to run this test is approx. 3*delayTime"



	Delay anyActive ifTrue:[

		^self notify:'Some delay is currently active.

Running this benchmark will not give any useful result.'].



	bb := Array new: 1. "The object we send the prim message to"



	"Compute the # of loops we'll run in a decent amount of time"

	[(Delay forMilliseconds: delayTime) wait] 

		forkAt: Processor userInterruptPriority.



	nLoops := 0.

	[Delay anyActive] whileTrue:[

		bb basicSize; basicSize; basicSize; basicSize; basicSize; 

			basicSize; basicSize; basicSize; basicSize; basicSize.

		nLoops := nLoops + 1.

	].



	"Flush the cache and make sure #basicSize is in there"

	Object flushCache.

	bb basicSize.



	"Now run the loop without any active delay

	for getting an idea about its actual speed."

	baseTime := self millisecondClockValue.

	index := nLoops.

	[index > 0] whileTrue:[

		bb basicSize; basicSize; basicSize; basicSize; basicSize; 

			basicSize; basicSize; basicSize; basicSize; basicSize.

		index := index - 1.

	].

	baseTime := self millisecondClockValue - baseTime.



	"Setup the active delay but try to never make it active"

	[(Delay forMilliseconds: delayTime + delayTime) wait] 

		forkAt: Processor userInterruptPriority.



	"And run the loop"

	actualTime := self millisecondClockValue.

	index := nLoops.

	[index > 0] whileTrue:[

		bb basicSize; basicSize; basicSize; basicSize; basicSize; 

			basicSize; basicSize; basicSize; basicSize; basicSize.

		index := index - 1.

	].

	actualTime := self millisecondClockValue - actualTime.



	"And get us some result"

	^((actualTime - baseTime) * 1000 asFloat / (nLoops * 10) truncateTo: 0.001) printString,

		' microseconds overhead per primitive call'
! !

!Time class methodsFor: 'general inquiries' stamp: 'nice 12/27/2009 03:11'!
condenseBunches: aCollectionOfSeconds
	| secArray now out pause prev bunchEnd |
	"Identify the major intervals in a bunch of numbers.  
	Each number is a seconds since 1901 that represents a date and time.
	We want the last event in a bunch.  Return array of seconds for:
	
	Every event in the last half hour.
		Every bunch separated by 30 min in the last 24 hours.
	
	Every bunch separated by two hours before that."

	"Time condenseBunches: 
		(#(20 400 401  20000 20200 20300 40000 45000  200000 201000 202000) 
			collect: [ :tt | self totalSeconds - tt])
"

	secArray := aCollectionOfSeconds asSortedCollection.
	pause := 1.
	now := self totalSeconds.
	out := OrderedCollection new.
	prev := 0.
	bunchEnd := nil.
	secArray reverseDo: [:secs | | ago | "descending"
		ago := now - secs.
		ago > (60*30) ifTrue: [pause := "60*30" 1800].
		ago > (60*60*24) ifTrue: [pause := "60*120" 7200].
		ago - prev >= pause ifTrue: [out add: bunchEnd.  bunchEnd := secs].
		prev := ago].
	out add: bunchEnd.
	out removeFirst.
	^ out
! !

!Time class methodsFor: 'general inquiries' stamp: 'brp 8/23/2003 23:59'!
humanWordsForSecondsAgo: secs
	| date today |
	"Return natural language for this date and time in the past."

	secs <= 1 ifTrue: [^ 'a second ago'].
	secs < 45 ifTrue: [^ secs printString, ' seconds ago'].
	secs < 90 ifTrue: [^ 'a minute ago'].
	secs < "45*60" 2700 ifTrue: [^ (secs//60) printString, ' minutes ago'].
	secs < "90*60" 5400 ifTrue: [^ 'an hour ago'].
	secs < "18*60*60" 64800 ifTrue: [^ (secs//3600) printString, ' hours ago'].
	date := Date fromSeconds: self totalSeconds - secs.		"now work with dates"
	today := Date today.
	date > (today subtractDays: 2) ifTrue: [^ 'yesterday'].
	date > (today subtractDays: 8) ifTrue: [^ 'last ', date dayOfWeekName].
	date > (today subtractDays: 13) ifTrue: [^ 'a week ago'].
	date > (today subtractDays: 28) ifTrue: [
		^ ((today subtractDate: date)//7) printString, ' weeks ago'].
	date > (today subtractDays: 45) ifTrue: [^ 'a month ago'].
	date > (today subtractDays: 300) ifTrue: [^ 'last ', date monthName].
	^ date monthName, ', ', date year printString

"Example
#(0.5 30 62 130 4000 10000 60000 90000 345600 864000 1728000 3456000 17280000 34560000 345600000) 
		collect: [:ss | Time humanWordsForSecondsAgo: ss].
"
! !

!Time class methodsFor: 'general inquiries' stamp: 'dtl 4/13/2011 08:06'!
millisecondClockMask
	"Answer the mask used for millisecond clock rollover in the virtual machine.
	Answer a default if the VM cannot supply the value."

	^MillisecondClockMask ifNil: [16r1FFFFFFF]
! !

!Time class methodsFor: 'general inquiries' stamp: 'nk 3/8/2004 12:05'!
millisecondClockValue
	"Answer the number of milliseconds since the millisecond clock was last reset or rolled over.
	Answer 0 if the primitive fails."

	<primitive: 135>
	^ 0! !

!Time class methodsFor: 'general inquiries' stamp: 'ar 11/25/2004 11:26'!
millisecondsToRun: timedBlock 
	"Answer the number of milliseconds timedBlock takes to return its value."

	| initialMilliseconds |
	initialMilliseconds := self millisecondClockValue.
	timedBlock value.
	^self millisecondsSince: initialMilliseconds! !

!Time class methodsFor: 'general inquiries' stamp: 'nice 12/27/2009 03:11'!
namesForTimes: arrayOfSeconds
	| simpleEnglish final prev |
	"Return English descriptions of the times in the array.  They are each seconds since 1901.  If two names are the same, append the date and time to distinguish them."

	simpleEnglish := arrayOfSeconds collect: [:secsAgo |
		self humanWordsForSecondsAgo: self totalSeconds - secsAgo].
	prev := ''.
	final := simpleEnglish copy.
	simpleEnglish withIndexDo: [:eng :ind | | prevPair myPair | 
		eng = prev ifFalse: [eng]
			ifTrue: ["both say 'a month ago'"
				prevPair := self dateAndTimeFromSeconds: 
						(arrayOfSeconds at: ind-1).
				myPair := self dateAndTimeFromSeconds: 
						(arrayOfSeconds at: ind).
				(final at: ind-1) = prev ifTrue: ["only has 'a month ago'"
					final at: ind-1 put: 
							(final at: ind-1), ', ', prevPair first mmddyyyy].
				final at: ind put: 
							(final at: ind), ', ', myPair first mmddyyyy.
				prevPair first = myPair first 
					ifTrue: [
						(final at: ind-1) last == $m ifFalse: ["date but no time"
							final at: ind-1 put: 
								(final at: ind-1), ', ', prevPair second printMinutes].
						final at: ind put: 
							(final at: ind), ', ', myPair second printMinutes]].
		prev := eng].
	^ final
! !

!Time class methodsFor: 'smalltalk-80' stamp: 'JZH 11/8/1998 13:03'!
dateAndTimeFromSeconds: secondCount

	^ Array
		with: (Date fromSeconds: secondCount)
		with: (Time fromSeconds: secondCount \\ 86400)! !

!Time class methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 19:50'!
dateAndTimeNow
	"Answer a two-element Array of (Date today, Time now)."

	^ self dateAndTimeFromSeconds: self totalSeconds
! !

!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 6/25/2009 07:43'!
fromSeconds: secondCount 
	"Answer an instance of me that is secondCount number of seconds since midnight."

	| integerSeconds nanos |
	integerSeconds := secondCount truncated.
	integerSeconds = secondCount
		ifTrue: [nanos := 0]
		ifFalse: [nanos := (secondCount - integerSeconds * NanosInSecond) asInteger].
	^ self seconds: integerSeconds nanoSeconds: nanos
! !

!Time class methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 20:01'!
new
	"Answer a Time representing midnight"

	^ self midnight! !

!Time class methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 22:01'!
primMillisecondClock
	"Primitive. Answer the number of milliseconds since the millisecond clock
	 was last reset or rolled over. Answer zero if the primitive fails.
	 Optional. See Object documentation whatIsAPrimitive."

	<primitive: 135>
	^ 0! !

!Time class methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 22:01'!
primSecondsClock
	"Answer the number of seconds since 00:00 on the morning of
	 January 1, 1901 (a 32-bit unsigned number).
	 Essential. See Object documentation whatIsAPrimitive. "

	<primitive: 137>
	self primitiveFailed! !

!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 17:34'!
readFrom: aStream
	"Read a Time from the stream in the form:
		<hour>:<minute>:<second> <am/pm>

	<minute>, <second> or <am/pm> may be omitted.  e.g. 1:59:30 pm; 8AM; 15:30"

	| hourMinuteSecondNano |
	hourMinuteSecondNano := self readHourMinuteSecondNanoFrom: aStream.
	^ self 
		hour: hourMinuteSecondNano first 
		minute: hourMinuteSecondNano second 
		second: hourMinuteSecondNano third 
		nanoSecond: hourMinuteSecondNano fourth

	"Time readFrom: (ReadStream on: '2:23:09 pm')"! !

!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 17:32'!
readHourMinuteSecondNanoFrom: aStream
	"Read a Time from the stream in the form:
		<hour>:<minute>:<second> <am/pm>

	<minute>, <second> or <am/pm> may be omitted.  e.g. 1:59:30 pm; 8AM; 15:30"

	| hour minute second ampm nanos nanosBuffer |
	hour := Integer readFrom: aStream.
	minute := 0.
	second := 0.
	nanosBuffer := '000000000' copy.
	nanos := WriteStream on: nanosBuffer.
	(aStream peekFor: $:) 
		ifTrue: [
			minute := Integer readFrom: aStream.
			(aStream peekFor: $:) 
				ifTrue: [
					second := Integer readFrom: aStream.
					(aStream peekFor: $.)
						ifTrue: [
							[aStream atEnd not and: [aStream peek isDigit]]
								whileTrue: [nanos nextPut: aStream next]]]].
	aStream skipSeparators.
	(aStream atEnd not and: ['PApa' includes: aStream peek]) ifTrue: 
		[ampm := aStream next asLowercase.
		(ampm = $p and: [hour < 12]) ifTrue: [hour := hour + 12].
		(ampm = $a and: [hour = 12]) ifTrue: [hour := 0].
		(aStream peekFor: $m) ifFalse: [aStream peekFor: $M ]].
	^ { hour . minute . second . nanosBuffer asInteger }

	"Time readFrom: (ReadStream on: '2:23:09 pm')"! !

!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 10/11/2004 22:15'!
totalSeconds
	"Answer the total seconds since the Squeak epoch: 1 January 1901."

	^ self primSecondsClock! !

!Time class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 19:46'!
current 

	^ self now! !

!Time class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:05'!
hour: hour minute: minute second: second
	"Answer a Time"

	^ self hour: hour minute: minute second: second nanoSecond: 0
! !

!Time class methodsFor: 'squeak protocol' stamp: 'brp` 8/24/2003 19:26'!
hour: hour minute: minute second: second  nanoSecond: nanoCount
	"Answer a Time - only second precision for now"

	^ self 
		seconds: (hour * SecondsInHour) + (minute * SecondsInMinute) + second 
		nanoSeconds: nanoCount
! !

!Time class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:01'!
midnight

	^ self seconds: 0
! !

!Time class methodsFor: 'squeak protocol' stamp: 'dtl 4/13/2011 08:05'!
milliseconds: currentTime since: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	| delta |
	delta := currentTime - lastTime.
	^ delta < 0
		ifTrue: [self millisecondClockMask + delta]
		ifFalse: [delta]! !

!Time class methodsFor: 'squeak protocol' stamp: 'mir 10/29/1999 18:24'!
millisecondsSince: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	^self milliseconds: self millisecondClockValue since: lastTime
! !

!Time class methodsFor: 'squeak protocol' stamp: 'brp` 8/24/2003 19:26'!
noon

	^ self seconds: (SecondsInDay / 2)
! !

!Time class methodsFor: 'squeak protocol' stamp: 'gk 8/31/2006 00:39'!
seconds: seconds
	"Answer a Time from midnight."

	^ self basicNew ticks: (Duration seconds: seconds) ticks! !

!Time class methodsFor: 'squeak protocol' stamp: 'gk 8/30/2006 23:29'!
seconds: seconds nanoSeconds: nanoCount
	"Answer a Time from midnight."

	^ self basicNew
		ticks: (Duration seconds: seconds nanoSeconds: nanoCount) ticks
! !

!Time class methodsFor: '*monticello' stamp: 'nk 11/2/2003 10:51'!
fromString: aString
	^ self readFrom: (ReadStream on: aString).
! !

!Time class methodsFor: 'clock' stamp: 'dtl 4/13/2011 08:09'!
initializeMillisecondClockMask
	"Initialize cached value from the VM, or set to nil if VM cannot support the request"

	MillisecondClockMask := self primMillisecondClockMask
! !

!Time class methodsFor: 'clock' stamp: 'ul 4/27/2013 22:00'!
primLocalMicrosecondClock
	"Answer the local microseconds since the Smalltalk epoch. The value is derived from the Posix epoch with a constant offset corresponding to elapsed microseconds between the two epochs according to RFC 868, and with an offset duration corresponding to the current offset of local time from UTC."

	<primitive: 241>
	^0! !

!Time class methodsFor: 'clock' stamp: 'dtl 8/14/2010 15:25'!
primMicrosecondClock
	"Answer the number of microseconds since the microsecond clock
	was last reset or rolled over. Answer zero if the primitive fails."

	<primitive: 'primitiveMicrosecondClock'>
	^ 0! !

!Time class methodsFor: 'clock' stamp: 'dtl 4/13/2011 08:02'!
primMillisecondClockMask
	"Answer the mask value used for millisecond clock rollover in the
	virtual machine, or nil if the VM cannot support the request."

	<primitive: 'primitiveMillisecondClockMask'>
	^nil! !

!Time class methodsFor: 'clock' stamp: 'dtl 4/28/2013 19:02'!
primPosixMicrosecondClockWithOffset
	"Answer an array with UTC microseconds since the Posix epoch and the
	current seconds offset from GMT in the local time zone."

	<primitive: 'primitiveUtcWithOffset'>
	^#(0 0)! !

!Time class methodsFor: 'clock' stamp: 'ul 4/27/2013 22:05'!
primUTCMicrosecondClock
	"Answer the UTC microseconds since the Smalltalk epoch. The value is derived from the Posix epoch with a constant offset corresponding to elapsed microseconds between the two epochs according to RFC 868."

	<primitive: 240>
	^0! !

!Time class methodsFor: 'clock' stamp: 'kph 12/14/2006 01:43'!
secondsWhenClockTicks

	"waits for the moment when a new second begins"

	| lastSecond delay |

	delay :=  Delay forMilliseconds: 1.
	lastSecond := self primSecondsClock.
	[ lastSecond = self primSecondsClock ] whileTrue: [ delay wait ].
 
	^ lastSecond + 1! !

!Time class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/9/2012 14:38'!
materializeFrom: aDecoder
	
	^ self 
		seconds: aDecoder nextEncodedUint24
		nanoSeconds: aDecoder nextEncodedUint32! !


!TimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 10:57'!
asDateAndTime
	"Answer the receiver as an instance of DateAndTime."

	^ DateAndTime utcMicroseconds: utcMicroseconds offset: localOffsetSeconds! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/16/2014 19:22'!
asTimeStamp
	"Answer the receiver as an instance of TimeStamp."

	^ self! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:14'!
date
	"Answer the date of the receiver."

	^ self asDate! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
dateAndTime
	"Answer a two element Array containing the receiver's date and time."

	^ Array with: self date with: self time! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
minusDays: anInteger
	"Answer a TimeStamp which is anInteger days before the receiver."

	^ self - (anInteger days)! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
minusSeconds: anInteger
	"Answer a TimeStamp which is anInteger number of seconds before the receiver."

	^ self - (anInteger seconds)! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:18'!
plusDays: anInteger
	"Answer a TimeStamp which is anInteger days after the receiver."

	^ self + (anInteger days)! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
plusSeconds: anInteger
	"Answer a TimeStamp which is anInteger number of seconds after the receiver."

	^ self + (anInteger seconds)! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
printOn: aStream 
	"Print receiver's date and time on aStream."

	aStream 
		nextPutAll: self date printString;
		space;
		nextPutAll: self time printString.! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
storeOn: aStream 

	aStream 
		print: self printString;
		nextPutAll: ' asTimeStamp'! !

!TimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:15'!
time
	"Answer the time of the receiver."

	^ self asTime! !

!TimeStamp methodsFor: '*Fuel' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^ aGeneralMapper visitFixedObject: self
! !


!TimeStamp class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 11:39'!
current

	^self now! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'nk 10/21/2003 23:07'!
fromMethodTimeStamp: aString
	| stream |
	stream := ReadStream on: aString.
	stream skipSeparators.
	stream skipTo: Character space.
	^self readFrom: stream.! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'cbc 2/4/2004 21:30'!
fromString: aString
	"Answer a new instance for the value given by aString.

	 TimeStamp fromString: '1-10-2000 11:55:00 am'. 
	"

	^self readFrom: (ReadStream on: aString).! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'nk 10/21/2003 23:04'!
readFrom: stream
	| date time |
	stream skipSeparators.
	date := Date readFrom: stream.
	stream skipSeparators.
	time := Time readFrom: stream.
	^self 
		date: date
		time: time! !


!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:28'!
abbreviation

	^ abbreviation! !

!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:28'!
abbreviation: aString

	abbreviation := aString! !

!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:29'!
name

	^ name! !

!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:28'!
name: aString

	name := aString! !

!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:28'!
offset

	^ offset
! !

!TimeZone methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:28'!
offset: aDuration

	offset := aDuration
! !

!TimeZone methodsFor: 'private' stamp: 'brp 9/4/2003 06:37'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self abbreviation;
		nextPut: $)! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/13/2004 12:18'!
dstTransitionTimes

	^ #()! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/8/2004 21:23'!
leapSecondsAt: aPointInTime
	"Ignore leap seconds"

	^ 0
! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/12/2004 10:31'!
localOffsetSecondsAt: aPointInTime

	^ self offset asSeconds! !


!TimeZone class methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:38'!
default
	"Answer the default time zone - GMT"

	^ self timeZones detect: [ :tz | tz offset = Duration zero ]! !

!TimeZone class methodsFor: 'accessing' stamp: 'cmm 3/6/2010 17:13'!
timeZones
	^{
		self offset:  0 hours name: 'Universal Time' abbreviation: 'UTC'.
		self offset:  0 hours name: 'Greenwich Mean Time' abbreviation: 'GMT'.
		self offset:  1 hours name: 'British Summer Time' abbreviation: 'BST'.
		self offset:  2 hours name: 'South African Standard Time' abbreviation: 'SAST'.
		self offset: -8 hours name: 'Pacific Standard Time' abbreviation: 'PST'.
		self offset: -7 hours name: 'Pacific Daylight Time' abbreviation: 'PDT'.
		self offset: -7 hours name: 'Mountain Standard Time' abbreviation: 'MST'.
		self offset: -6 hours name: 'Mountain Daylight Time' abbreviation: 'MDT'.
		self offset: -6 hours name: 'Central Standard Time' abbreviation: 'CST'.
		self offset: -5 hours name: 'Central Daylight Time' abbreviation: 'CDT'.
		self offset: -5 hours name: 'Eastern Standard Time' abbreviation: 'EST'.
		self offset: -4 hours name: 'Eastern Daylight Time' abbreviation: 'EDT'.
	}! !

!TimeZone class methodsFor: 'instance creation' stamp: 'brp 9/4/2003 06:33'!
offset: aDuration name: aName abbreviation: anAbbreviation

	^ self new
		offset: aDuration;
		name: aName;
		abbreviation: anAbbreviation;
		yourself
! !

!TimeZone class methodsFor: '*time-TimeZones' stamp: 'dtl 8/21/2007 23:14'!
setLocal: tzName
	"Set the local time zone to a time zone identified by tzName. To identify
	suitable time zone names for e.g. Detriot, search the database using:
		TimeZoneDatabase systemDatabase grepFor: 'Detroit'"

	"TimeZone setLocal: 'CET'"
	"TimeZone setLocal: 'EST'"
	"TimeZone setLocal: 'America/Detroit'"
	"TimeZone setLocal: 'no such name'"

	(TimeZoneDatabase systemDatabase timeZoneFor: tzName)
		ifNil: [self notify: '''', tzName asString,
				''' not found in TimeZoneDatabase class>>systemDatabase']
		ifNotNil: [TimeZoneProxy setDynamicTransform: tzName].
	^ TimeZoneDatabase systemDatabase timeZoneFor: tzName
	
! !


!Timespan methodsFor: 'ansi protocol' stamp: 'brp 9/15/2003 14:05'!
+ operand
	"operand conforms to protocol Duration"
	

	^ self class starting: (self start + operand) duration: self duration! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 9/15/2003 14:07'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asDateAndTime)

	 	ifTrue: [ self start - operand ]
	
	ifFalse: [ self + (operand negated) ]
! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:43'!
< comparand

	^ self start < comparand! !

!Timespan methodsFor: 'ansi protocol' stamp: 'nice 1/1/2010 21:49'!
= comparand
	^ self class = comparand class 
		and: [ self start = comparand start
		and: [ self duration = comparand duration ] ]
.! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 7/27/2003 17:49'!
dayOfMonth
	"Answer the day of the month represented by the receiver."

	^ start dayOfMonth! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 8/6/2003 18:42'!
dayOfWeek
	"Answer the day of the week represented by the receiver."

	^ start dayOfWeek! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 8/6/2003 18:42'!
dayOfWeekName
	"Answer the day of the week represented by the receiver."

	^ start dayOfWeekName! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 8/24/2003 11:50'!
dayOfYear
	"Answer the day of the year represented by the receiver."

	^ start dayOfYear! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:44'!
hash

	^ start hash + duration hash! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:44'!
isLeapYear

	^ start isLeapYear! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:44'!
month

	^ start month! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 1/7/2004 16:25'!
monthAbbreviation


	^ start monthAbbreviation! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:44'!
monthName


	^ start monthName! !

!Timespan methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 08:44'!
year


	^ start year! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:49'!
dates


	| dates |

	dates := OrderedCollection new.
	self datesDo: [ :m | dates add: m ].
	^ dates asArray! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:49'!
datesDo: aBlock


	self do: aBlock with: start asDate
! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:50'!
every: aDuration do: aBlock

	| element end |
	element := self start.
	end := self end.
	[ element <= end ] whileTrue:
	
	[ aBlock value: element.
		element := element + aDuration. ]! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:50'!
months

	| months |
	months := OrderedCollection new: 12.
	self monthsDo: [ :m | months add: m ].
	^ months asArray.
! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:50'!
monthsDo: aBlock

	self do: aBlock with: start asMonth! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:51'!
weeks


	| weeks |
	weeks := OrderedCollection new.
	self weeksDo: [ :m | weeks add: m ].
	^ weeks asArray! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:51'!
weeksDo: aBlock

	self do: aBlock with: self asWeek.! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:51'!
workDatesDo: aBlock
	"Exclude Saturday and Sunday"

	self do: aBlock with: start asDate when: [ :d | d dayOfWeek < 6 ]
! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:51'!
years


	| years |
	years := OrderedCollection new.
	self yearsDo: [ :m | years add: m ].
	^ years asArray! !

!Timespan methodsFor: 'enumerating' stamp: 'brp 5/13/2003 08:58'!
yearsDo: aBlock

	self do: aBlock with: start asYear.! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 14:09'!
day
	"Answer the day of the year represented by the receiver."
	^ self dayOfYear! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 08:45'!
daysInMonth


	^ start daysInMonth! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 08:45'!
daysInYear
	"Answer the number of days in the month represented by the receiver."

	^ start daysInYear! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 17:50'!
daysLeftInYear
	^ start daysLeftInYear! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 17:55'!
firstDayOfMonth

	^ start firstDayOfMonth! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 08:47'!
monthIndex

	^ self month! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 08:47'!
next

	^ self class starting: (start + duration) duration: duration! !

!Timespan methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 08:48'!
previous


	^ self class starting: (start - duration) duration: duration! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:44'!
asDate


	^ start asDate! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:44'!
asDateAndTime

	^ start! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/30/2003 00:10'!
asDuration

	^ self duration! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:45'!
asMonth


	^ start asMonth! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:45'!
asTime

	^ start asTime! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:25'!
asTimeStamp

	^ start asTimeStamp! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:45'!
asWeek

	^ start asWeek! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:45'!
asYear


	^ start asYear
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:45'!
duration
	"Answer the Duration of this timespan"

	^ duration! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 9/23/2004 09:53'!
end


	^ self duration asNanoSeconds = 0
		ifTrue: [ self start ]
		ifFalse: [ self next start - DateAndTime clockPrecision ]
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 1/7/2004 16:05'!
includes: aDateAndTime


	^ (aDateAndTime isKindOf: Timespan)
			ifTrue: [ (self includes: aDateAndTime start)
						and: [ self includes: aDateAndTime end ] ]
			ifFalse: [ aDateAndTime asDateAndTime between: start and: self end ]! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:54'!
includesAllOf: aCollection 
	"Answer whether all the elements of aCollection are in the receiver."

	aCollection do: [:elem | (self includes: elem) ifFalse: [^ false]].
	^ true
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 1/7/2004 15:59'!
includesAnyOf: aCollection 
	"Answer whether any element of aCollection is included in the receiver"

	aCollection do: [ :elem | (self includes: elem) ifTrue: [^ true]].
	^false
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:47'!
intersection: aTimespan

	 "Return the Timespan both have in common, or nil"

	 | aBegin anEnd |
	 aBegin := self start max: aTimespan start.
	 anEnd := self end min: aTimespan end.
	 anEnd < aBegin ifTrue: [^nil].

	 ^ self class starting: aBegin ending: anEnd
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:47'!
julianDayNumber


	^ start julianDayNumber! !

!Timespan methodsFor: 'squeak protocol' stamp: 'cmm 7/10/2011 21:31'!
makeUTC
	"Change the receiver's timezone to UTC, which affords substantially better hashing performance."
	start makeUTC! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 09:17'!
printOn: aStream


	super printOn: aStream.
	aStream 
		nextPut: $(;
		print: start;
		nextPut: $D;
		print: duration;
		nextPut: $)
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:48'!
start
	"Answer the start DateAndTime of this timespan"

	^ start! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:48'!
start: aDateAndTime
	"Store the start DateAndTime of this timespan"

	start := aDateAndTime asDateAndTime! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:49'!
to: anEnd
	"Answer an Timespan. anEnd must be aDateAndTime or a Timespan"


	^ Timespan starting: (self start) ending: (anEnd asDateAndTime)
! !

!Timespan methodsFor: 'squeak protocol' stamp: 'brp 1/9/2004 16:46'!
union: aTimespan
	 "Return the Timespan spanned by both"

	| aBegin anEnd |

	aBegin := self start min: aTimespan start.
	anEnd := self end max: aTimespan end.
	^ Timespan starting: aBegin ending: (anEnd + DateAndTime clockPrecision)
! !

!Timespan methodsFor: 'private' stamp: 'brp 5/13/2003 08:58'!
do: aBlock with: aFirstElement

	self do: aBlock with: aFirstElement when: [ :t | true ]
! !

!Timespan methodsFor: 'private' stamp: 'brp 5/13/2003 08:59'!
do: aBlock with: aFirstElement when: aConditionBlock

	| element end |
	element := aFirstElement.
	end := self end.
	[ element start <= end ] whileTrue:
	
	[(aConditionBlock value: element)
			ifTrue: [ aBlock value: element ].
		element := element next. ]
! !

!Timespan methodsFor: 'private' stamp: 'brp 5/13/2003 08:59'!
duration: aDuration
	"Set the Duration of this timespan"

	duration := aDuration! !

!Timespan methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 22:58'!
maPrintAbbreviatedOn: aStream
	self printOn: aStream! !

!Timespan methodsFor: '*magma-client' stamp: 'cmm 10/28/2010 10:41'!
isImmutableInMagma 
	"These don't change once they're created, so don't bother to record them in magma's readSet."
	^ true! !


!Date methodsFor: 'printing' stamp: 'brp 7/27/2003 16:07'!
mmddyyyy
	"Answer the receiver rendered in standard U.S.A format mm/dd/yyyy.
	Note that the name here is slightly misleading -- the month and day numbers don't show leading zeros, 
	so that for example February 1 1996 is 2/1/96"


	^ self printFormat: #(2 1 3 $/ 1 1)
! !

!Date methodsFor: 'printing' stamp: 'ul 8/12/2014 18:32'!
printFormat: formatArray 
	"Answer a String describing the receiver using the argument formatArray."

	^String new: 17 streamContents: [ :stream |
		self printOn: stream format: formatArray ]! !

!Date methodsFor: 'printing' stamp: 'BP 5/18/2000 18:05'!
printOn: aStream

	self printOn: aStream format: #(1 2 3 $  3 1 )
! !

!Date methodsFor: 'printing' stamp: 'brp 7/27/2003 16:05'!
printOn: aStream format: formatArray 
	"Print a description of the receiver on aStream using the format 
	denoted the argument, formatArray: 
	
		#(item item item sep monthfmt yearfmt twoDigits) 
	
		items: 1=day 2=month 3=year will appear in the order given, 
	
		separated by sep which is eaither an ascii code or character. 
	
		monthFmt: 1=09 2=Sep 3=September 
	
		yearFmt: 1=1996 2=96 
	
		digits: (missing or)1=9 2=09. 
	
	See the examples in printOn: and mmddyy"
	| gregorian twoDigits element monthFormat |
	gregorian := self dayMonthYearDo: [ :d :m :y | {d. m. y} ].
	twoDigits := formatArray size > 6 and: [(formatArray at: 7) > 1].
	1 to: 3 do: 
		[ :i | 
			element := formatArray at: i.
			element = 1
				ifTrue: [twoDigits
						ifTrue: [aStream
								nextPutAll: (gregorian first asString
										padded: #left
										to: 2
										with: $0)]
						ifFalse: [gregorian first printOn: aStream]].
			element = 2
				ifTrue: [monthFormat := formatArray at: 5.
					monthFormat = 1
						ifTrue: [twoDigits
								ifTrue: [aStream
										nextPutAll: (gregorian middle asString
												padded: #left
												to: 2
												with: $0)]
								ifFalse: [gregorian middle printOn: aStream]].
					monthFormat = 2
						ifTrue: [aStream
								nextPutAll: ((Month nameOfMonth: gregorian middle)
										copyFrom: 1
										to: 3)].
					monthFormat = 3
						ifTrue: [aStream
								nextPutAll: (Month nameOfMonth: gregorian middle)]].
			element = 3
				ifTrue: [(formatArray at: 6)
							= 1
						ifTrue: [gregorian last printOn: aStream]
						ifFalse: [aStream
								nextPutAll: ((gregorian last \\ 100) asString
										padded: #left
										to: 2
										with: $0)]].
			i < 3
				ifTrue: [(formatArray at: 4)
							~= 0
						ifTrue: [aStream nextPut: (formatArray at: 4) asCharacter]]]! !

!Date methodsFor: 'printing' stamp: 'di 9/22/2000 12:47'!
storeOn: aStream

	aStream print: self printString; nextPutAll: ' asDate'
! !

!Date methodsFor: 'printing' stamp: 'brp 7/27/2003 16:04'!
yyyymmdd
	"Format the date in ISO 8601 standard like '2002-10-22'."

	^ self printFormat: #(3 2 1 $- 1 1 2)
! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 22:09'!
addDays: dayCount 

	^ (self asDateAndTime + (dayCount days)) asDate! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:08'!
asSeconds
	"Answer the seconds since the Squeak epoch: 1 January 1901"

	^ start asSeconds
! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:08'!
leap
	"Answer whether the receiver's year is a leap year."

	^ start isLeapYear ifTrue: [1] ifFalse: [0].! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 1/16/2004 14:30'!
previous: dayName 
	"Answer the previous date whose weekday name is dayName."

	| days |
	days := 7 + self weekdayIndex - (self class dayOfWeek: dayName) \\ 7.
	days = 0 ifTrue: [ days := 7 ].
	^ self subtractDays: days! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:09'!
subtractDate: aDate 
	"Answer the number of days between self and aDate"

	^ (self start - aDate asDateAndTime) days! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 22:05'!
subtractDays: dayCount 

	^ (self asDateAndTime - (dayCount days)) asDate! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 8/24/2003 12:04'!
weekday
	"Answer the name of the day of the week on which the receiver falls."

	^ self dayOfWeekName! !

!Date methodsFor: 'smalltalk-80' stamp: 'brp 8/24/2003 12:04'!
weekdayIndex
	"Sunday=1, ... , Saturday=7"

	^ self dayOfWeek! !

!Date methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 16:10'!
asDate

	^ self
! !

!Date methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 16:10'!
dayMonthYearDo: aBlock 
	"Supply integers for day, month and year to aBlock and return the result"

	^ start dayMonthYearDo: aBlock! !

!Date methodsFor: 'squeak protocol' stamp: 'avi 2/21/2004 18:12'!
month
	^ self asMonth! !

!Date methodsFor: 'squeak protocol' stamp: 'avi 2/29/2004 13:10'!
monthIndex
	^ super month! !

!Date methodsFor: 'utils' stamp: 'tbn 7/11/2006 10:30'!
addMonths: monthCount 
	|year month maxDaysInMonth day |
	year := self year + (monthCount + self monthIndex - 1 // 12).
	month := self monthIndex + monthCount - 1 \\ 12 + 1.
	maxDaysInMonth := Month daysInMonth: month forYear: year.
	day := self dayOfMonth > maxDaysInMonth
				ifTrue: [maxDaysInMonth]
				ifFalse: [self dayOfMonth].
	^ Date
		newDay: day
		month: month
		year: year! !

!Date methodsFor: 'utils' stamp: 'spfa 3/8/2004 13:52'!
onNextMonth

	^ self addMonths: 1
! !

!Date methodsFor: 'utils' stamp: 'spfa 3/8/2004 13:52'!
onPreviousMonth

	^ self addMonths: -1
! !

!Date methodsFor: '*Fuel' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^ aGeneralMapper visitHookPrimitive: self
! !

!Date methodsFor: '*time-UTC' stamp: 'dtl 11/25/2012 12:25'!
asDateInTimeZone: aTimeZone
	"The start time for a Date varies according to time zone, and various versions
	of Squeak handle Date instance creation differently. Answer a new instance
	with start time appropriate to aTimeZone."

	| approximateStartTime approximateOffset approximateOffsetSeconds actualOffsetSeconds actualStartTime |
	"Values are approximate because a DST transition might occur near this time"
	approximateStartTime := PointInTime
								fromSmalltalkSeconds: self asSeconds
								inTimeZone: aTimeZone.
	approximateOffsetSeconds := aTimeZone localOffsetSecondsAt: approximateStartTime.
	approximateOffset := Duration seconds: approximateOffsetSeconds.
	start := DateAndTime
				year: self year
				day: self day
				hour: 0
				minute: 0
				second: 0
				offset: approximateOffset.
	actualStartTime := PointInTime
							fromSmalltalkSeconds: start asSeconds
							inTimeZone: aTimeZone.
	actualOffsetSeconds := aTimeZone localOffsetSecondsAt: actualStartTime.
	actualOffsetSeconds = approximateOffsetSeconds ifFalse: [ | actualOffset |
		"Revise the calculation if a DST transition has affected the estimate"
		actualOffset := Duration seconds: actualOffsetSeconds.
		start := DateAndTime
					year: self year
					day: self day
					hour: 0
					minute: 0
					second: 0
					offset: actualOffset].
	^ self class starting: start
	

! !

!Date methodsFor: '*FuelCompatibilityBeforeSqueak46' stamp: 'MaxLeske 2/17/2013 22:43'!
serializeOn: anEncoder
	start serializeOn: anEncoder! !

!Date methodsFor: '*ma-core' stamp: 'cmm 7/29/2008 23:14'!
maAddYears: anInteger 
	"Answer a new date anInteger years after or, if anInteger is negative, before, the receiver."
	^ Date 
		newDay: self dayOfMonth
		month: self monthIndex
		year: self year + anInteger! !

!Date methodsFor: '*ma-core' stamp: 'cmm 6/17/2010 15:29'!
maSuggestedFilename 
	^ self yyyymmdd asFileName! !


!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:04'!
asMonth

	^ self! !

!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:05'!
daysInMonth

	^ self duration days.! !

!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:05'!
index

	^ self monthIndex! !

!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:05'!
name


	^ self monthName! !

!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:05'!
previous


	^ self class starting: (self start - 1)! !

!Month methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:05'!
printOn: aStream


	aStream nextPutAll: self monthName, ' ', self year printString! !

!Month methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 23:04'!
maPrintAbbreviatedOn: aStream 
	aStream
		 maPrint: self year asString ;
		 maPrint: $- ;
		 maPrint:
			(self monthIndex
				printPaddedWith: $0
				to: 2)! !

!Month methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 23:33'!
maSuggestedFilename
	"Often, it is useful for month filenames to be sortable, therefore override the standard maSuggestedFilename which would otherwise answer, 'July', 'August', etc."
	^ String streamContents: [ : stream | self maPrintAbbreviatedOn: stream ]! !


!Schedule methodsFor: 'enumerating' stamp: 'brp 5/13/2003 09:50'!
between: aStart and: anEnd do: aBlock

	| element end i |
	end := self end min: anEnd.
	element := self start.
	
	i := 1.
	[ element < aStart ] whileTrue:
	
	[ element := element + (schedule at: i).
		i := i + 1. (i > schedule size) ifTrue: [i := 1]].
	i := 1.
	[ element <= end ] whileTrue:
	
	[ aBlock value: element.
		element := element + (schedule at: i).
		i := i + 1.
		(i > schedule size) ifTrue: [i := 1]]
! !

!Schedule methodsFor: 'enumerating' stamp: 'brp 5/13/2003 09:50'!
dateAndTimes

	| dateAndTimes |
	dateAndTimes := OrderedCollection new.
	self scheduleDo: [ :e | dateAndTimes add: e ].
	^ dateAndTimes asArray! !

!Schedule methodsFor: 'enumerating' stamp: 'brp 5/13/2003 09:50'!
schedule
	^ schedule! !

!Schedule methodsFor: 'enumerating' stamp: 'brp 5/13/2003 09:50'!
schedule: anArrayOfDurations

	schedule := anArrayOfDurations! !

!Schedule methodsFor: 'enumerating' stamp: 'brp 5/13/2003 09:51'!
scheduleDo: aBlock

	self between: (self start) and: (self end) do: aBlock
! !

!Schedule methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:38'!
includes: aDateAndTime

	| dt |
	dt := aDateAndTime asDateAndTime.
	self scheduleDo: [ :e | e = dt ifTrue: [^true] ].
	^ false
! !


!Timespan class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 20:19'!
current
	^ self starting: (DateAndTime nowWithOffset: self defaultOffset)! !

!Timespan class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 22:08'!
new
	"Answer a Timespan starting on the Squeak epoch: 1 January 1901"
	^ self starting: (DateAndTime new offset: self defaultOffset)! !

!Timespan class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 08:42'!
starting: aDateAndTime


	^ self starting: aDateAndTime duration: Duration zero! !

!Timespan class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 18:48'!
starting: aDateAndTime duration: aDuration

	^ self basicNew
 		start: aDateAndTime asDateAndTime;
		duration: aDuration;
		yourself! !

!Timespan class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 16:16'!
starting: startDateAndTime ending: endDateAndTime

	^ self 
		starting: startDateAndTime 
		duration: (endDateAndTime asDateAndTime - startDateAndTime)
! !

!Timespan class methodsFor: 'configuring' stamp: 'cmm 2/10/2012 16:51'!
defaultOffset
	"Timespans created in the context of an offset will start in that offset.  When no context is available, the defaultOffset for Timespans must be zero.  For example, two ways to make a Date for today:
	Date today.  'start is midnight at offset zero.  Will compare successfully to other Date today results.'
	DateAndTime now asDate.  'In this case, the start is midnight of the local time-zone.  It can only compare equally to Dates of its time-zone.'"
	^ Duration zero! !


!Date class methodsFor: 'general inquiries' stamp: 'BG 3/16/2005 14:57'!
easterDateFor: year

 "  compute the easter date.
    source: Physikalisch-Technische Bundesanstalt Braunschweig.
    Lichtenberg, H.: Zur Interpretation der Gaussschen Osterformel
                     und ihrer Ausnahmeregeln,
                     Historia Mathematica 24 (1997), pp. 441-444
  
    http://www.ptb.de/de/org/4/44/441/oste.htm
  "

  | k m s a d r og sz oe day |

  k := year // 100.
  m := 15 + (3*k + 3//4) - (8*k + 13//25).
   s := 2 - (3*k + 3// 4).
  a := year \\ 19.
  d := 19*a + m \\ 30.
  r := d//29 + ((d//28) - (d//29)* (a// 11)).

  og := 21 + d - r.
  sz := 7 - (year//4 + year + s\\7).
  oe := 7 - (og - sz\\7).
  day := og + oe.
  ^day <= 31
    ifTrue: [Date newDay: day month: 3 year: year ]
    ifFalse: [Date newDay: day - 31 month: 4 year: year].! !

!Date class methodsFor: 'general inquiries' stamp: 'BG 3/16/2005 14:48'!
orthodoxEasterDateFor: year

 "  compute the easter date according to the rules of the orthodox calendar.
    source: 
    http://www.smart.net/~mmontes/ortheast.html 
  "
     | r1 r2 r3 r4 ra rb r5 rc date |

    r1 := year \\ 19.
    r2 := year \\ 4.
    r3 := year \\ 7.
    ra := 19*r1 + 16.
    r4 := ra \\ 30.
    rb := r2 + r2 + (4*r3) + (6*r4).
    r5 := rb \\ 7.
    rc := r4 + r5.
    date := Date newDay: 3 month: 4 year: year.
    ^date addDays: rc.! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 8/24/2003 00:00'!
dateAndTimeNow
	"Answer an Array whose with Date today and Time now."

	^ Time dateAndTimeNow
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:35'!
dayOfWeek: dayName 

	^ Week indexOfDay: dayName! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:59'!
daysInMonth: monthName forYear: yearInteger 

	^ Month daysInMonth: monthName forYear: yearInteger.
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:53'!
daysInYear: yearInteger 

	^ Year daysInYear: yearInteger.! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 1/16/2004 14:35'!
firstWeekdayOfMonth: month year: year
	"Answer the weekday index of the first day in <month> in the <year>."

	^ (self newDay: 1 month: month year: year) weekdayIndex
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:01'!
fromDays: dayCount 
	"Days since 1 January 1901"

	^ self julianDayNumber: dayCount + SqueakEpoch! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:02'!
fromSeconds: seconds
	"Answer an instance of me which is 'seconds' seconds after January 1, 1901."

	^ self fromDays: ((Duration seconds: seconds) days)
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:39'!
indexOfMonth: aMonthName 

	^ Month indexOfMonth: aMonthName.
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:56'!
leapYear: yearInteger 

	^ Year leapYear: yearInteger! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:37'!
nameOfDay: dayIndex 

	^ Week nameOfDay: dayIndex ! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:40'!
nameOfMonth: anIndex 

	^ Month nameOfMonth: anIndex.
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:02'!
newDay: day month: month year: year 

	^ self year: year month: month day: day
! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:01'!
newDay: dayCount year: yearInteger

	^ self year: yearInteger day: dayCount! !

!Date class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:01'!
today

	^ self current
! !

!Date class methodsFor: 'squeak protocol' stamp: 'md 7/15/2006 18:06'!
fromString: aString
	"Answer an instance of created from a string with format mm.dd.yyyy."

	^ self readFrom: aString readStream.! !

!Date class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:49'!
julianDayNumber: aJulianDayNumber 
	^ self starting:
		(DateAndTime
			julianDayNumber: aJulianDayNumber
			offset: self defaultOffset)! !

!Date class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:01'!
readFrom: aStream 
	"Read a Date from the stream in any of the forms:  
		<day> <month> <year>		(15 April 1982; 15-APR-82; 15.4.82; 15APR82)  
		<month> <day> <year>		(April 15, 1982; 4/15/82)
		<year>-<month>-<day>			(1982-04-15) (ISO8601)"
	| ymd |
	ymd := self readYearMonthDayFrom: aStream.
	^ self
		year: ymd first
		month: ymd second
		day: ymd third
! !

!Date class methodsFor: 'squeak protocol' stamp: 'Igor.Stasenko 1/15/2010 14:04'!
readFrom: inputStream pattern: pattern
	"Read a Date from the stream based on the pattern which can include the tokens:
	
		y = A year with 1-n digits
		yy = A year with 2 digits
		yyyy = A year with 4 digits
		m = A month with 1-n digits
		mm = A month with 2 digits
		d = A day with 1-n digits
		dd = A day with 2 digits
		
	...and any other Strings inbetween. Representing $y, $m and $d is done using
	\y, \m and \d and slash itself with \\. Simple example patterns:

		'yyyy-mm-dd'
		'yyyymmdd'
		'yy.mm.dd'
		'y-m-d'

	A year given using only two decimals is considered to be >2000."

	| day month year patternStream char |
	patternStream := pattern readStream.
	[patternStream atEnd] whileFalse: [
		inputStream atEnd ifTrue: [^nil].
		char := patternStream next.
		char = $\
			ifTrue: [inputStream next = patternStream next ifFalse: [^nil]]
			ifFalse: [
				char = $y
					ifTrue: [
						(patternStream nextMatchAll: 'yyy')
							ifTrue: [year := (inputStream next: 4) asInteger]
							ifFalse: [
								(patternStream peekFor: $y)
									ifTrue: [
										year := (inputStream next: 2) asInteger]
									ifFalse: [
										year := Integer readFrom: inputStream]]]
					ifFalse: [
						char = $m
							ifTrue: [
								(patternStream peekFor: $m)
									ifTrue: [
										month := (inputStream next: 2) asInteger]
									ifFalse: [
										month := Integer readFrom: inputStream]]
							ifFalse: [
								char = $d
									ifTrue: [
										(patternStream peekFor: $d)
											ifTrue: [
												day := (inputStream next: 2) asInteger]
											ifFalse: [
												day := Integer readFrom: inputStream]]
									ifFalse: [
										inputStream next = char ifFalse: [^nil]]]]]].
	(year isNil | month isNil | day isNil) ifTrue: [^nil].
	^self year: year month: month day: day! !

!Date class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 16:57'!
readYearMonthDayFrom: aStream 
	"Read Date information from the stream in any of the forms:  
		<day> <month> <year>		(15 April 1982; 15-APR-82; 15.4.82; 15APR82)  
		<month> <day> <year>		(April 15, 1982; 4/15/82)
		<year>-<month>-<day>			(1982-04-15) (ISO8601)"
	| day month year parsedNumber prefix monthIndex |
	aStream peek = $-
		ifTrue: [prefix := -1]
		ifFalse: [prefix := 1].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	aStream peek isDigit
		ifTrue: [
			parsedNumber := (Integer readFrom: aStream) * prefix.
			(parsedNumber < 0 or: [parsedNumber > 31])
				ifTrue: [year := parsedNumber]].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	aStream peek isLetter
		ifTrue: ["MM-DD-YY or DD-MM-YY or YY-MM-DD"
			month := WriteStream on: (String new: 10).
			[aStream peek isLetter]
				whileTrue: [month nextPut: aStream next].
			month := month contents.
			[aStream peek isAlphaNumeric]
				whileFalse: [aStream skip: 1].
			parsedNumber isNil
				ifTrue: ["MM DD YY"
					day := Integer readFrom: aStream]
				ifFalse: [
					year isNil
						ifTrue: ["DD MM YY"
							day := parsedNumber]]]
		ifFalse: ["MM-DD-YY or DD-MM-YY or YY-MM-DD"
			year isNil 
				ifTrue: ["MM-DD-YY or DD-MM-YY"
					parsedNumber > 12
						ifTrue: ["DD-MM-YY"
							day := parsedNumber.
							monthIndex := Integer readFrom: aStream.
							"month := Month nameOfMonth: (Integer readFrom: aStream)"]
						ifFalse: ["MM-DD-YY"
							monthIndex := parsedNumber.
							"month := Month nameOfMonth: parsedNumber."
							day := Integer readFrom: aStream]]
				ifFalse: ["YY-MM-DD"
					monthIndex := Integer readFrom: aStream.
					"month := Month nameOfMonth: (Integer readFrom: aStream)"]].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	year isNil
		ifTrue: [year := Integer readFrom: aStream]
		ifFalse: [day := Integer readFrom: aStream].
	(year < 100 and: [year >= 0]) 
		ifTrue: [
			year < 69 
				ifTrue: [	year := 2000 + year]
				ifFalse: [year := 1900 + year]].

	monthIndex ifNil: [monthIndex := Month indexOfMonth: month].
	^ { year . monthIndex . day }
! !

!Date class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:18'!
starting: aDateAndTime 
	^ self
		starting: aDateAndTime midnight
		duration: Duration oneDay! !

!Date class methodsFor: 'squeak protocol' stamp: 'brp 7/1/2003 18:09'!
tomorrow

	^ self today next! !

!Date class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:50'!
year: year day: dayOfYear 
	^ self starting:
		(DateAndTime
			year: year
			day: dayOfYear
			hour: 0
			minute: 0
			second: 0
			offset: self defaultOffset)! !

!Date class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:50'!
year: year month: month day: day 
	^ self starting:
		(DateAndTime
			year: year
			month: month
			day: day
			hour: 0
			minute: 0
			second: 0
			offset: self defaultOffset)! !

!Date class methodsFor: 'squeak protocol' stamp: 'brp 7/1/2003 18:09'!
yesterday

	^ self today previous! !

!Date class methodsFor: '*FuelCompatibilityBeforeSqueak46' stamp: 'MaxLeske 2/17/2013 22:47'!
materializeFrom: aDecoder
	^ self 
		starting: (DateAndTime materializeFrom: aDecoder)
		duration: (Duration days: 1)! !


!Month class methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 16:27'!
daysInMonth: indexOrName forYear: yearInteger 

	| index |
	index := indexOrName isInteger 
				ifTrue: [indexOrName]
				ifFalse: [self indexOfMonth: indexOrName].
	^ (DaysInMonth at: index)
			+ ((index = 2
					and: [Year isLeapYear: yearInteger])
						ifTrue: [1] ifFalse: [0])
! !

!Month class methodsFor: 'smalltalk-80' stamp: 'brp 8/23/2003 09:29'!
indexOfMonth: aMonthName


	1 to: 12 do: [ :i |  (aMonthName, '*' match: (MonthNames at: i)) ifTrue: [^i] ].
	self error: aMonthName , ' is not a recognized month name'.! !

!Month class methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 09:02'!
nameOfMonth: anIndex

	^ MonthNames at: anIndex.! !

!Month class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 20:16'!
month: month year: year 
	"Create a Month for the given <year> and <month>.
	<month> may be a number or a String with the
	name of the month. <year> should be with 4 digits."
	^ self starting:
		(DateAndTime
			year: year
			month: month
			day: 1
			hour: 0
			minute: 0
			second: 0
			offset: self defaultOffset)! !

!Month class methodsFor: 'squeak protocol' stamp: 'nice 2/23/2013 21:02'!
readFrom: aStream
	| m y |
	aStream skipSeparators.
	m := aStream upToAnyOf: CharacterSet separators.
	aStream skipSeparators.
	y := aStream upToEnd.
	^ self 
		month: m
		year: y asInteger

"Month readFrom: 'July 1998' readStream"! !

!Month class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 23:15'!
starting: aDateAndTime duration: aDuration 
	"Override - a each month has a defined duration"
	| start adjusted days |
	start := aDateAndTime asDateAndTime.
	adjusted := DateAndTime
		year: start year
		month: start month
		day: 1
		hour: 0
		minute: 0
		second: 0
		offset: start offset.
	days := self
		daysInMonth: adjusted month
		forYear: adjusted year.
	^ super
		starting: adjusted
		duration: (Duration days: days)! !


!Week methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:36'!
asWeek

	^ self! !

!Week methodsFor: 'squeak protocol' stamp: 'brp 1/30/2005 09:31'!
index

	^ self asMonth dayOfWeek + self dayOfMonth - 2  // 7 + 1
! !

!Week methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:37'!
printOn: aStream

	aStream nextPutAll: 'a Week starting: '.
	self start printOn: aStream
! !


!Week class methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 09:34'!
nameOfDay: anIndex

	^ DayNames at: anIndex! !

!Week class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:34'!
dayNames

	^ DayNames! !

!Week class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:34'!
indexOfDay: aSymbol

	^ DayNames indexOf: aSymbol! !

!Week class methodsFor: 'squeak protocol' stamp: 'brp 1/30/2005 09:17'!
startDay

	^ StartDay ifNil: [ StartDay
 := DayNames first ]
! !

!Week class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 09:30'!
startDay: aSymbol

	(DayNames includes: aSymbol)
		ifTrue: [ StartDay := aSymbol ]
		ifFalse: [ self error: aSymbol, ' is not a recognised day name' ]! !

!Week class methodsFor: 'squeak protocol' stamp: 'gk 8/30/2006 15:10'!
starting: aDateAndTime duration: aDuration
	"Override - the duration is always one week.
	 Week will start from the Week class>>startDay"

	| midnight delta adjusted |
	midnight := aDateAndTime asDateAndTime midnight.
	delta := ((midnight dayOfWeek + 7 - (DayNames indexOf: self startDay)) rem: 7) abs.
	adjusted := midnight - (Duration days: delta seconds: 0).

	^ super starting: adjusted duration: (Duration weeks: 1)! !


!Year methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:01'!
asYear


	^ self! !

!Year methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:01'!
daysInMonth


	self shouldNotImplement! !

!Year methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:01'!
daysInYear

	^ self duration days.! !

!Year methodsFor: 'squeak protocol' stamp: 'brp 5/21/2003 08:38'!
printOn: aStream

	aStream nextPutAll: 'a Year ('.
	self start year printOn: aStream.

	aStream nextPutAll: ')'
! !

!Year methodsFor: 'smalltalk-80' stamp: 'brp 6/16/2008 08:41'!
previous
	"This implementation handles leap years correctly"
	
	^ self class year: (self year - 1)! !


!Year class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:52'!
daysInYear: yearInteger

	^ 365 + ((self isLeapYear: yearInteger) ifTrue: [1] ifFalse: [0]).
! !

!Year class methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 13:55'!
leapYear: yearInteger 

	^ (self isLeapYear: yearInteger)
		ifTrue: [1]
		ifFalse: [0]! !

!Year class methodsFor: 'squeak protocol' stamp: 'brp 9/11/2003 14:05'!
current
 
	^ self year: (DateAndTime now year)
! !

!Year class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 09:00'!
isLeapYear: aYearInteger


	| adjustedYear |
	adjustedYear := aYearInteger > 0
		ifTrue: [aYearInteger]
		ifFalse: [(aYearInteger + 1) negated].

	"There was no year 0"
	^ ((adjustedYear \\ 4 ~= 0) or: [(adjustedYear \\ 100 = 0) and: [adjustedYear \\ 400 ~= 0]]) not! !

!Year class methodsFor: 'squeak protocol' stamp: 'brp 7/1/2003 13:53'!
starting: aDateAndTime duration: aDuration 
	"Override - start from midnight"
	| midnight |
	midnight := aDateAndTime asDateAndTime midnight.

	^ super
		starting: midnight
		duration: (Duration days: (self daysInYear: midnight year))! !

!Year class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 20:21'!
year: aYear 
	^ self starting:
		(DateAndTime
			year: aYear
			month: 1
			day: 1
			hour: 0
			minute: 0
			second: 0
			offset: self defaultOffset)! !

Magnitude subclass: #Time
	instanceVariableNames: 'seconds nanos'
	classVariableNames: 'MillisecondClockMask'
	poolDictionaries: 'ChronologyConstants'
	category: 'Chronology-Core'!
Duration initialize!
DateAndTime initialize!
ChronologyConstants initialize!
SharedPool subclass: #ChronologyConstants
	instanceVariableNames: 'seconds offset jdn nanos'
	classVariableNames: 'DayNames DaysInMonth MonthNames NanosInMillisecond NanosInSecond OneDay SecondsInDay SecondsInHour SecondsInMinute SqueakEpoch Zero'
	poolDictionaries: ''
	category: 'Chronology-Core'!
"Postscript:
Initialize as at image startup time"

DateAndTime initializeOffsets.

!

